#! /usr/bin/env python

"""
Script for performing a fit to a histogramm of recorded 
time differences for the use with QNet
"""


import scipy.optimize as optimize
import numpy
import pylab
import sys
import optimalbins
import datetime
import os
import shutil
import time
from subprocess import Popen

def main(bincontent=None):

    def decay(p,x):
        return p[0]*numpy.exp(-x/p[1])+p[2]
    def error(p,x,y):
        return decay(p,x)-y
    
    if bincontent == None:
    
        nbins = float(sys.argv[3])
        xmin = float(sys.argv[2])
        xmax = 10.0
        #xmin = 2.0
        
        #!!!the correction factor 0.75/1.25 is due to the different tmc (0.75 and 1.25 ns) and only required for the fermi samples!!! 
        times = [float(l) for l in open(sys.argv[1]).readlines() if xmin < float(l) < xmax]
        
        print len(times),"decay times"
        #times = float(times)
        #nbins = optimalbins.optbinsize(times,1,30)
        print "Nbins:",nbins
        date = time.gmtime()
        bin_edges = numpy.linspace(xmin,xmax,nbins)
        bin_centers = bin_edges[:-1] + 0.5*(bin_edges[1]-bin_edges[0])
        hist,edges = numpy.histogram(times,bin_edges)
        
        #hist=hist[:-1]
        p0 = numpy.array([200,2.0,5])
        
        output = optimize.leastsq(error,p0,args=(bin_centers,hist),full_output=1)
        p = output[0]
        covar = output[1]
        
        print "Fit parameters:",p
        print "Covariance matrix:",covar
        
        chisquare=0.
        deviations=error(p,bin_centers,hist)
        for i,d in enumerate(deviations):
            chisquare += d*d/decay(p,bin_centers[i])
        
        params = {'legend.fontsize': 10}
        params = {'legend.textsize': 4}
        pylab.rcParams.update(params)
        
        fitx=numpy.linspace(xmin,xmax,100)
        pylab.plot(bin_centers,hist,"b^",fitx,decay(p,fitx),"b-")
        pylab.xlim(0,11)
        pylab.ylim(0,max(hist)+20)
        pylab.xlabel("Decay time [$\mu$s]")
        pylab.ylabel("Events in time bin")
        #pylab.legend(("Data","Fit: (%4.2f +- %4.2f) microsec,$s^2$/ndf=%4.2f"%(p[1],numpy.sqrt(covar[1][1]),chisquare/(nbins-len(p)))))

        pylab.legend(("Data","Lifetime: (%4.2f $\pm$ %4.2f) $\mu$s,   $\chi^2$/ndf=%4.2f"%(p[1],numpy.sqrt(covar[1][1]),chisquare/(nbins-len(p)))))
        #pylab.errorbar()
        pylab.grid()
   
        # this is hard-coded! There must be a better solution...
        # if you change here, you have to change in setup.py!
        datapath = os.getenv('HOME') + os.sep + 'muonic_data'
         

        date = datetime.datetime.now()
        pylab.savefig(os.path.join(datapath,"muon_lifetime_%s.png" %(%date.strftime('%Y-%m-%d_%H-%M-%S'))))

        pylab.savefig(os.path.join(datapath,"muon_lifetime_%s.pdf" %(%date.strftime('%Y-%m-%d_%H-%M-%S'))))

        #FIXME: should think about invoking p.show() here...
        p=Popen("eog " + os.path.join(datapath,"muon_lifetime%s.png" %(%date.strftime('%Y-%m-%d_%H-%M-%S'))), shell=True) 
       

    else:

        # this is then used for the mudecaywindow
        # in muonic
        # we have to adjust the bins
        # to the values of the used histogram

        bins = numpy.linspace(0,20,84)
        bin_centers = bins[:-1] + 0.5*(bins[1]-bins[0])


        p0 = numpy.array([200,2.0,5])

        #try: 
        print bincontent

        output = optimize.leastsq(error,p0,args=(bin_centers,bincontent),full_output=1)
        #except:
        #    print "Fit failed!"
        #    return

        p = output[0]
        covar = output[1]
        
        print "Fit parameters:",p
        print "Covariance matrix:",covar
        
        chisquare=0.
        deviations=error(p,bin_centers,bincontent)
        for i,d in enumerate(deviations):
            chisquare += d*d/decay(p,bin_centers[i])
        
        params = {'legend.fontsize': 13}
        pylab.rcParams.update(params)
        
        nbins = 84
        xmin = bin_centers[0]
        xmax = bin_centers[-1]

        fitx=numpy.linspace(xmin,xmax,100)

        return (bin_centers,bincontent,fitx,decay,p,covar,chisquare,nbins)
        
        #pylab.plot(bin_centers,bincontent,"b^",fitx,decay(p,fitx),"b-")
        #pylab.ylim(0,max(bincontent)+100)
        #pylab.xlabel("Decay time in microseconds")
        #pylab.ylabel("Events in time bin")
        #pylab.legend(("Data","Fit: (%4.2f +- %4.2f) microsec,chisq/ndf=%4.2f"%(p[1],numpy.sqrt(covar[1][1]),chisquare/(nbins-len(p)))))
        #pylab.savefig("fit.png")
        
        # vim: ai ts=4 sts=4 et sw=4


if __name__ == '__main__':
    main()



#! /usr/bin/env python

import sys

# what frequency is the correct one?
# 41 or 25?
# assuming 41..
#freq = 25.0e6
freq = 41.0e6

def time_to_seconds(time, correction):
    '''
    Convert hhmmss,xxx string int seconds since day start
    '''
#    print time,correction
    tfields = time.split(".")
    t = tfields[0]
    secs_since_day_start = int(t[0:2])*3600+int(t[2:4])*60+int    (t[4:6])
    evt_time = secs_since_day_start + int(tfields[1])/1000.0+int(correction)/1000.0
    #return round(evt_time)
    return evt_time

def get_time(fields,last_onepps_count):
    try:
        trigger_count = int(fields[0],16)
        onepps_count = int(fields[9],16)
    except IndexError:
        return None
    if onepps_count != last_onepps_count:
        last_onepps_count = onepps_count
    time = fields[10]
    correction = fields[15]
    
    line_time = time_to_seconds(time,correction)+(trigger_count - onepps_count)/freq

#line_time = seconds + (trigger_count - onepps_coun    t)/freq     
    return line_time


if __name__ == '__main__':

    last_onepps_count = 0
    f = open(sys.argv[1])
    ini = True
    for line in f.readlines():
            line = line.split()
            try:
                    if ini:
                            start = get_time(line,last_onepps_count)
                            ini= False
    
                    stop = get_time(line,last_onepps_count)
            except ValueError:
                    print 'Error',line
                    pass
    print start
    print (stop - start)/3600



onesize = 20

def LargeScreenMPStyle(): 
    """ 
    Large fonts for large screens
    """ 
    from pylab import rc

    rc("lines",linewidth=2,markersize=10)
    rc("axes",titlesize=onesize,labelsize=onesize)
    rc("grid",linewidth=1.0)
    rc("xtick",labelsize=onesize)
    rc("xtick.major",size=7)
    rc("xtick.minor",size=5)
    rc("ytick",labelsize=onesize)
    rc("ytick.major",size=7)
    rc("ytick.minor",size=5)
    rc("grid",linewidth=1.2)
    #rc("font",serif="Palatino")
    fontprops = dict()
    fontprops["size"] = onesize
    fontprops["family"] = "TeX Gyre Pagella" # this is Palatino

    rc("font",**fontprops)
    rc("legend",fontsize=onesize,markerscale=1,numpoints=1)

    rc("ps" ,useafm = True)
    rc("pdf" , use14corefonts = True)
    rc("text",usetex=True) 
    rc("xtick",labelsize=onesize)
    rc("ytick",labelsize=onesize)
    rc("font",size =  onesize)



"""
Provide the canvases for plots in muonic
"""

import matplotlib.pyplot as mp
import sys
import pylab as p
import numpy as n
from datetime import datetime

# Python Qt4 bindings for GUI objects
from PyQt4 import QtGui

# Matplotlib Figure object
from matplotlib.figure import Figure

# import the Qt4Agg FigureCanvas object, that binds Figure to
# Qt4Agg backend. It also inherits from QWidget
from matplotlib.backends.backend_qt4agg \
import FigureCanvasQTAgg as FigureCanvas

# import the NavigationToolbar Qt4Agg widget
from matplotlib.backends.backend_qt4agg \
import NavigationToolbar2QTAgg as NavigationToolbar

class MuonicPlotCanvas(FigureCanvas):
    """
    The base class of all muonic plot canvases
    """
    
    def __init__(self,parent,logger,ymin=0,ymax=10,xmin=0,xmax=10,xlabel="xlabel",ylabel="ylabel",grid=True, spacing=(0.1,0.9)):
       
        self.logger = logger
        
        self.fig = Figure(facecolor="white",dpi=72)
        self.ax = self.fig.add_subplot(111)
        self.fig.subplots_adjust(left=spacing[0], right=spacing[1])
  
        # initialization of the canvas
        FigureCanvas.__init__(self, self.fig)

        # set specific limits for X and Y axes
        #print ymin,ymax
        self.ax.set_ylim(ymin=ymin,ymax=ymax)
        self.ax.set_xlim(xmin=xmin,xmax=xmax)
        self.ax.set_xlabel(xlabel)
        self.ax.set_ylabel(ylabel)
        self.ax.set_autoscale_on(False)
        self.ax.grid(grid)
        
        #store the limits for later 
        self.xmin = xmin
        self.xmax = xmax
        self.ymin = ymin
        self.ymax = ymax
        self.xlabel = xlabel
        self.ylabel = ylabel
        
        # force a redraw of the Figure
        self.fig.canvas.draw() 
        self.setParent(parent)


    def color(self, string, color="none"):
        """
        output colored strings on the terminal
        """
        colors = { "green": '\033[92m', 'yellow' : '\033[93m', 'red' : '\033[91m', 'blue' : '\033[94m', 'none' : '\033[0m'}
        return colors[color] + string + colors["none"]   


    def update_plot(self):
        """
        Instructions to updated this plot
        implement this individually
        """
        self.logger.warning("Update plot is not implemented for this Canvas!")
        
        
class PulseCanvas(MuonicPlotCanvas):
    """
    Matplotlib Figure widget to display Pulses
    """

    def __init__(self,parent,logger):
        super(PulseCanvas,self).__init__(parent,logger,ymin=0,ymax=1.2,xmin=0,xmax=40,xlabel="Time (ns)",ylabel="ylabel",grid=True)
        self.ax.yaxis.set_visible(False)   
        self.ax.set_title("Oscilloscope") 
  
    def update_plot(self, pulses):
      
        #do a complete redraw of the plot to avoid memory leak!
        self.ax.clear()

        # set specific limits for X and Y axes
        self.ax.set_xlim(0, 100)
        self.ax.set_ylim(ymax=1.5)
        self.ax.grid()
        self.ax.set_xlabel('Time (ns)')
        self.ax.yaxis.set_visible(False)
        self.ax.set_title("Oscilloscope")
        # and disable figure-wide autoscale
        self.ax.set_autoscale_on(False)

        # we have only the information that the pulse is over the threshold,
        # besides that we do not have any information about its height
        # TODO: It would be nice to implement the thresholds as scaling factors

        self.pulseheight = 1.0

        colors = ['b','g','r','c']
        labels = ['c0','c1','c2','c3']
        _pulsemax = []
        if pulses is None:
            self.logger.warning('Pulses have no value - channels not connected?')
        else:
            for chan in enumerate(pulses[1:]):
                for pulse in chan[1]:
                    self.ax.plot([pulse[0],pulse[0],pulse[1],pulse[1]],[0,self.pulseheight,self.pulseheight,0],colors[chan[0]],label=labels[chan[0]],lw=2)
                    _pulsemax.append(pulse[0])
                    _pulsemax.append(pulse[1])

            _pulsemax = max(_pulsemax)*1.2
            # TODO: the trick below does not really work as expected. 
            #if _pulsemax < self.ax.get_xlim()[1]:
            #    _pulsemax = self.ax.get_xlim()[0]
            self.ax.set_xlim(0, _pulsemax)
            try:
                self.ax.legend(loc=1, ncol=5, mode="expand", borderaxespad=0., handlelength=1)
            except:
                self.logger.info('An error with the legend occured!')
                self.ax.legend(loc=2)

            self.fig.canvas.draw()
        
        
class ScalarsCanvas(MuonicPlotCanvas):
    
    def __init__(self,parent,logger, MAXLENGTH = 40):
        
        MuonicPlotCanvas.__init__(self,parent,logger,ymin=0, ymax=20,xlabel="Time (s)",ylabel="Rate (1/s)")
        self.do_not_show_trigger = False
        #max length of shown = MAXLENGTH*timewindow
        self.MAXLENGTH = MAXLENGTH
        self.reset()
        
    def reset(self,show_pending = False):
        """reseting all data"""

        self.ax.clear()
        self.ax.grid()
        self.ax.set_xlabel(self.xlabel)
        self.ax.set_ylabel(self.ylabel)
        self.ax.set_xlim((self.xmin, self.xmax))
        self.ax.set_ylim((self.ymin, self.ymax))

        # and disable figure-wide autoscale
        #self.ax.set_autoscale_on(False)
        self.highest=0
        self.lowest=0
        self.now = datetime.now()#.strftime('%d.%m.%Y %H:%M:%S')
        #self.ax.set_xlim(0., 5.2)
        #self.ax.set_ylim(0., 100.2)
        
        self.timewindow = 0
        self.chan0, self.chan1, self.chan2, self.chan3, self.trigger, self.l_time =[], [], [], [], [], []
        self.l_chan0, = self.ax.plot(self.l_time,self.chan0, c='y', label='ch0',lw=3)
        self.l_chan1, = self.ax.plot(self.l_time,self.chan1, c='m', label='ch1',lw=3)
        self.l_chan2, = self.ax.plot(self.l_time,self.chan2, c='c',  label='ch2',lw=3)
        self.l_chan3, = self.ax.plot(self.l_time,self.chan3, c='b', label='ch3',lw=3)
        if not self.do_not_show_trigger:
            self.l_trigger, = self.ax.plot(self.l_time,self.trigger, c='g', label='trigger',lw=3)

        self.N0 = 0
        self.N1 = 0
        self.N2 = 0
        self.N3 = 0
        self.NT = 0

        if show_pending:
            left, width = .25, .5
            bottom, height = .35, .8
            right = left + width
            top = bottom + height
            #pr_ax = self.ax.reshape(-1)[0]
            self.ax.text(0.5*(left+right), 0.5*(bottom+top), 'Measuring...',
            horizontalalignment='center',
            verticalalignment='center',
            fontsize=56, color='red',
            fontweight="heavy",
            alpha=.8,
            rotation=30,
            transform=self.fig.transFigure)

        self.fig.canvas.draw()

    def update_plot(self, result, trigger = False,channelcheckbox_0 = True,channelcheckbox_1 = True,channelcheckbox_2 = True,channelcheckbox_3 = True):

        #do a complete redraw of the plot to avoid memory leak!
        self.ax.clear()
        self.do_not_show_trigger = trigger
        # set specific limits for X and Y axes
        #self.ax.set_xlim(0., 5.2)
        #self.ax.set_ylim(0., 100.2)
        self.ax.grid()
        self.ax.set_xlabel(self.xlabel)
        self.ax.set_ylabel(self.ylabel)

        # and disable figure-wide autoscale
        #self.ax.set_autoscale_on(False)
        self.logger.debug("result : %s" %result.__repr__())

        # update lines data using the lists with new data
        self.chan0.append(result[0])
        self.chan1.append(result[1])
        self.chan2.append(result[2])
        self.chan3.append(result[3])
        self.trigger.append(result[4])
        self.timewindow += result[5]
        self.l_time.append(self.timewindow)
        self.N0 += result[6]
        self.N1 += result[7]
        self.N2 += result[8]
        self.N3 += result[9]
        self.NT += result[10]
        if channelcheckbox_0:
            self.l_chan0, = self.ax.plot(self.l_time,self.chan0, c='y', label='ch0',lw=2,marker='v')
        if channelcheckbox_1:
            self.l_chan1, = self.ax.plot(self.l_time,self.chan1, c='m', label='ch1',lw=2,marker='v')
        if channelcheckbox_2:
            self.l_chan2, = self.ax.plot(self.l_time,self.chan2, c='c', label='ch2',lw=2,marker='v')
        if channelcheckbox_3:
            self.l_chan3, = self.ax.plot(self.l_time,self.chan3, c='b', label='ch3',lw=2,marker='v')
        if not self.do_not_show_trigger:
            self.l_trigger, = self.ax.plot(self.l_time,self.trigger, c='g', label='trg',lw=2,marker='x')

        try:
            _ncol_cnt = 0
            for ch_chk_bx in [channelcheckbox_0,channelcheckbox_1,channelcheckbox_2,channelcheckbox_3]:
                if ch_chk_bx:
                    _ncol_cnt += 1

            if not self.do_not_show_trigger:
                _ncol_cnt += 1
            
            self.ax.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3, ncol=_ncol_cnt, mode="expand", borderaxespad=0., handlelength=2)
            
        except:
            self.logger.info('An error with the legend occured!')
            self.ax.legend(loc=2)

        if len(self.chan0) > self.MAXLENGTH:
            self.chan0.remove(self.chan0[0])
            self.chan1.remove(self.chan1[0])
            self.chan2.remove(self.chan2[0])
            self.chan3.remove(self.chan3[0])
            self.trigger.remove(self.trigger[0])
            self.l_time.remove(self.l_time[0])

      
        self.logger.debug("self.l_chan0: %s" %self.l_chan0.__repr__())
        
        self.logger.debug("Chan0 to plot: %s", self.chan0.__repr__())
        
        ma = max( max(self.chan0), max(self.chan1), max(self.chan2),max(self.chan3), max(self.trigger)  )
            
        self.ax.set_ylim(0, ma*1.1)
        self.ax.set_xlim(self.l_time[0], self.l_time[-1])

        now2 = datetime.now()
        dt = (now2 - self.now)  
        dt1 = dt.seconds + dt.days * 3600 * 24

        self.fig.canvas.draw()

class MuonicHistCanvas(MuonicPlotCanvas):
    """
    A base class for all canvases with a histogram
    """
     
    def __init__(self,parent,logger,binning,histcolor="b",**kwargs): 
        #super(MuonicHistCanvas,self).__init__(self,parent,logger,**kwargs)
        MuonicPlotCanvas.__init__(self,parent,logger,**kwargs)
        self.binning = binning
        self.bincontent   = n.zeros(len(binning-1))
        self.hist_patches = self.ax.hist(n.array([self.binning[0]-1]), 
                                         self.binning, fc=histcolor, 
                                         alpha=0.25)[2]
        self.heights = []
        self.underflow = 0 #FIXME the current implementation does not know about outliers
        self.overflow  = 0 #FIXME the current implementation does not know about outliers
        self.dimension = r"$\mu$s"
        # fixed xrange for histogram
        self.xmin      = binning[0]
        self.xmax      = binning[-1] + (binning[:-1] - binning[1:])[-1]  
        
    def update_plot(self,data):    

        if not data:
            return None

        # avoid memory leak
        self.ax.clear()

        _max_xval = self.ax.get_xlim()[1]
        if not data is None:
            if len(data) > 0:
                _max_xval = 1.2*max(data)

        # we have to do some bad hacking here,
        # because the p histogram is rather
        # simple and it is not possible to add
        # two of them...
        # however, since we do not want to run into a memory leak
        # and we also be not dependent on dashi (but maybe
        # sometimes in the future?) we have to do it
        # by manipulating rectangles...

        # we want to find the non-empty bins
        # tmphist is compatible with the decaytime hist...
        tmphist = self.ax.hist(data, self.binning, fc="b", alpha=0.25)[0]

        for histbin in enumerate(tmphist):
            if histbin[1]:
                self.hist_patches[histbin[0]].set_height(self.hist_patches[histbin[0]].get_height() + histbin[1])
            else:
                pass

        # we want to get the maximum for the ylims
        # self.heights contains the bincontent!

        self.heights = []
        for patch in self.hist_patches:
            self.heights.append(patch.get_height())

        self.logger.debug('Histogram patch heights %s' %self.heights.__repr__())
        self.ax.set_ylim(ymax=max(self.heights)*1.1)
        self.ax.set_ylim(ymin=0)
        self.ax.set_xlabel(self.xlabel)
        self.ax.set_ylabel(self.ylabel)

        # FIXED: we don't want to cut off something from 
        # the histogram, so make it just large
        # FIXME: adjust the size properly
        # HOTFIX: disabled, does not seem to work at all
        #if  _max_xval*5. > self.ax.get_xlim()[1]:
        #    _max_xval = self.ax.get_xlim()[1]
        #self.ax.set_xlim(xmax=_max_xval)
        self.ax.set_xlim(xmin=self.xmin,xmax=self.xmax)
        # always get rid of unused stuff
        del tmphist

        # some beautification
        self.ax.grid()

        # we now have to pass our new patches 
        # to the figure we created..            
        self.ax.patches = self.hist_patches 
        self.fig.canvas.draw()


    def show_fit(self,bin_centers,bincontent,fitx,decay,p,covar,chisquare,nbins):

        #self.ax.clear()
        #print decay(p,fitx)
        #print p
        # clears a previous fit from the canvas 
        self.ax.lines = []
        self.ax.plot(bin_centers,bincontent,"b^",fitx,decay(p,fitx),"b-")
        #FIXME: this seems to crop the histogram
        #self.ax.set_ylim(0,max(bincontent)*1.2)
        self.ax.set_xlabel(self.xlabel)
        self.ax.set_ylabel(self.ylabel)
        # compute the errors on the fit, nb that this calculation assumes that
        # scipy.optimize.leastsq was used
        error = []
        for i in range(len(p)):
            try:
              error.append( n.absolute(covar[i][i])**0.5)
            except:
              error.append( 0.00 )
        perr_leastsq = n.array(error)
        try:
            #self.ax.legend(("Data","Fit: (%4.2f +- %4.2f) %s  \n chisq/ndf=%4.2f"%(p[1],n.sqrt(covar[1][1]),self.dimension,chisquare/(nbins-len(p)))),loc=1)
            self.ax.legend(("Data","Fit: (%4.2f $\pm$ %4.2f) %s  \n chisq/ndf=%4.2f"%(p[1],perr_leastsq[2],self.dimension,chisquare/(nbins-len(p)))),loc=1)
        except TypeError:
            self.logger.warn('Covariance Matrix is None, could not calculate fit error!')
            self.ax.legend(("Data","Fit: (%4.2f) %s \n chisq/ndf=%4.2f"%(p[1],self.dimension,chisquare/(nbins-len(p)))),loc=1)
                   
        self.fig.canvas.draw()


class LifetimeCanvas(MuonicHistCanvas):
    """
    A simple histogram for the use with mu lifetime
    measurement
    """
    
    def __init__(self,parent,logger,binning = (0,10,21)): 
        MuonicHistCanvas.__init__(self,parent,logger,n.linspace(binning[0],binning[1],binning[2]),xlabel="Time between Pulses ($\mu$s)",ylabel="Events")

     
class VelocityCanvas(MuonicHistCanvas):  
    
    def __init__(self,parent,logger,binning = (0.,30,25)): 
        MuonicHistCanvas.__init__(self,parent,logger,n.linspace(binning[0],binning[1],binning[2]),xmin=0.,xmax=30,ymin=0,ymax=2,ylabel="Events",xlabel="Flight Time (ns)") 
        self.dimension = r"$ns$"
        
class PulseWidthCanvas(MuonicHistCanvas):     

    def __init__(self,parent,logger,histcolor="r"): 
        MuonicHistCanvas.__init__(self,parent,logger,n.linspace(0.,100,30),histcolor=histcolor,xmin=0.,xmax=100,ymin=0,ymax=2,ylabel="Events",xlabel="Pulse Width (ns)") 
        self.ax.set_title("Pulse widths")
        
    def update_plot(self,data):
        super(PulseWidthCanvas,self).update_plot(data)
        self.ax.set_title("Pulse widths")
        
        



#! /usr/bin/env python

import Queue
import serial
import os
import subprocess 

try:
    import zmq
except ImportError:
    print "No zmq installed..."


import os.path as pth
from time import sleep

class DaqConnection(object):

    def __init__(self, inqueue, outqueue, logger):

        self.inqueue = inqueue
        self.outqueue = outqueue
        self.running = 1
        self.logger = logger

        try:
            self.port = self.get_port()
        except serial.SerialException, e:
            self.logger.fatal("SerialException thrown! Value:" + e.message.__repr__())
            raise SystemError, e

    def get_port(self):
        """
        check out which device (/dev/tty) is used for DAQ communication
        """
        connected = False
        while not connected:
            #which_tty_daq = os.path.split(os.path.abspath(__file__))[0] + os.sep + "which_tty_daq"

            try:
                dev = subprocess.Popen(["which_tty_daq"], stdout=subprocess.PIPE).communicate()[0]
            except OSError: # using package script
                which_tty_daq = os.path.abspath('./bin/which_tty_daq')
                if not os.path.exists(which_tty_daq):
                    raise OSError("Can not find binary which_tty_daq")
                dev = subprocess.Popen([which_tty_daq], stdout=subprocess.PIPE).communicate()[0]

            dev = "/dev/" + dev
            dev = dev.rstrip('\n')
            self.logger.info("Daq found at %s",dev)
            self.logger.info("trying to connect...")
            try:
                port = serial.Serial(port=dev, baudrate=115200,
                                     bytesize=8,parity='N',stopbits=1,
                                     timeout=0.5,xonxoff=True)
                connected = True
            except serial.SerialException, e:
                self.logger.error(e)
                self.logger.error("Waiting 5 seconds")
                sleep(5)

        self.logger.info("Successfully connected to serial port")
        return port



    def read(self):
        """
        Get data from the DAQ. Read it from the provided Queue.
        """
        min_sleeptime = 0.01 # seconds
        max_sleeptime = 0.2 # seconds
        sleeptime = min_sleeptime #seconds
        while self.running:
#            data = self.port.read(1)
#            n = self.port.inWaiting()
#            if n > 0:
#                data += self.port.read(n)
#            for line in data.split('\n'):
#                self.outqueue.put(line)
            try:
                if self.port.inWaiting():
                    while self.port.inWaiting():
                        self.outqueue.put(self.port.readline().strip())
                    sleeptime = max(sleeptime/2, min_sleeptime)
                else:
                    sleeptime = min(1.5 * sleeptime, max_sleeptime)
                sleep(sleeptime)

            except IOError:
                self.logger.error("IOError")
                self.port.close()
                self.port = self.get_port()
                # this has to be implemented in the future
                # for now, we assume that the card does not forget
                # its settings, only because the USB connection is
                # broken
                #self.setup_daq.setup(self.commandqueue)
            except OSError:
                self.logger.error("IOError")
                self.port.close()
                self.port = self.get_port()
                #self.setup_daq.setup(self.commandqueue)



    def write(self):
        """
        Put messages from the inqueue which is filled by the DAQ
        """

        while self.running:
            try:
                while self.inqueue.qsize():
                    try:
                        self.port.write(str(self.inqueue.get(0))+"\r")
                    except Queue.Empty:
                        pass
                
                sleep(0.1)
            except NotImplementedError:
                self.logger.debug("Running Mac version of muonic.")
                while True:
                    try:
                        self.port.write(str(self.inqueue.get(timeout=0.01))+"\r")
                    except Queue.Empty:
                        pass
                 
                sleep(0.1)


class DaqServer(DaqConnection):
    
    # FIXME make constructors consistent
    def __init__(self, port,logger):


        self.running = 1
        self.logger = logger
        self.setup_socket(port)
        try:
            self.port = self.get_port()
        except serial.SerialException, e:
            self.logger.fatal("SerialException thrown! Value:" + e.message.__repr__())
            raise SystemError, e

    
    
    def setup_socket(self,port,adress="127.0.0.1"):
        #port = "5556"
        context = zmq.Context()
        self.socket = context.socket(zmq.PAIR)
        self.socket_port = port
        self.socket_adress = adress
        self.socket.bind("tcp://%s:%s" %(adress,port))

    def serve(self):
        while True:
            self.read()
            self.write()

    def read(self):
        """
        Get data from the DAQ. Read it from the provided Queue.
        """
        min_sleeptime = 0.01 # seconds
        max_sleeptime = 0.2 # seconds
        sleeptime = min_sleeptime #seconds
        while self.running:
#            data = self.port.read(1)
#            n = self.port.inWaiting()
#            if n > 0:
#                data += self.port.read(n)
#            for line in data.split('\n'):
#                self.outqueue.put(line)
            try:
                if self.port.inWaiting():
                    while self.port.inWaiting():
                        self.socket.send(self.port.readline().strip())
                    sleeptime = max(sleeptime/2, min_sleeptime)
                else:
                    sleeptime = min(1.5 * sleeptime, max_sleeptime)
                sleep(sleeptime)

            except IOError:
                self.logger.error("IOError")
                self.port.close()
                self.port = self.get_port()
                # this has to be implemented in the future
                # for now, we assume that the card does not forget
                # its settings, only because the USB connection is
                # broken
                #self.setup_daq.setup(self.commandqueue)
            except OSError:
                self.logger.error("IOError")
                self.port.close()
                self.port = self.get_port()
                #self.setup_daq.setup(self.commandqueue)

    def write(self):
        """
        Put messages from the inqueue which is filled by the DAQ
        """

        while self.running:
            msg = self.socket.recv_string()
            self.port.write(str(msg)+"\r")
            sleep(0.1)



if __name__ == "__main__":
    
    import logging
    logger = logging.getLogger()
    x = DaqServer("5556",logger)
    x.serve()



"""
Provide the dialog fields for user interaction
"""


from PyQt4 import QtCore, QtGui

class MuonicDialog(QtGui.QDialog):
    """
    Base class of all muonic dialogs
    """
    
    def __init__(self):
        pass
    
    def createButtonBox(self,objectname="buttonBox",leftoffset=80,topoffset=900):
        """
        Create a custom button for cancel/apply
        """
        
        buttonBox = QtGui.QDialogButtonBox(self)
        buttonBox.setGeometry(QtCore.QRect(leftoffset, topoffset, 300, 32))
        buttonBox.setOrientation(QtCore.Qt.Horizontal)
        buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        buttonBox.setObjectName(objectname)
        return buttonBox

    def createCheckGroupBox(self,label="Single Pulse",objectname="singlecheckbox",radio=False,leftoffset=20,setchecked=None,checkable=False, checkable_set=False,itemlabels=["Chan0","Chan1","Chan2","Chan3"]):
        """
        Create a group of choices
        """

        groupBox = QtGui.QGroupBox(label)
        groupBox.setCheckable(checkable)
        groupBox.setChecked(checkable_set)
        groupBox.setObjectName(objectname)

        checkitems = []
        for index,label in enumerate(itemlabels):
            
            if radio:
                check = QtGui.QRadioButton(self)
            else:
                check = QtGui.QCheckBox(self)
                
            check.setGeometry(QtCore.QRect(leftoffset, 40 + index*40, 119, 28))
            check.setObjectName(objectname + "_" + str(index))
            check.setText(label) 
            checkitems.append(check)        
 
        if setchecked is not None:
            for index in setchecked:
                checkitems[index].setChecked(True)
                
        #for channel in enumerate(checkitems):
        #    if checkitem == channel[0]:
        #        channel[1].setChecked(True)
            
        vbox = QtGui.QVBoxLayout()
        for check in checkitems:
            vbox.addWidget(check)
        
        vbox.addStretch(1)
        groupBox.setLayout(vbox)
        return groupBox

class DecayConfigDialog(MuonicDialog):
    """
    Settings for the muondecay
    """
    
    def __init__(self, *args):

        QtGui.QDialog.__init__(self,*args)

        # size of window etc..
        self.setObjectName("Configure")
        #self.resize(480, 360)
        self.setModal(True)
        self.setWindowTitle("Muon Decay Configuration")  

        grid = QtGui.QGridLayout()
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Single Pulse",objectname = "singlecheckbox",leftoffset=20, setchecked=[1]), 0, 0)
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Double Pulse",objectname = "doublecheckbox",leftoffset=180,setchecked=[2]), 0, 1)
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Software Veto Channel",objectname = "vetocheckbox",leftoffset=300,    setchecked=[3]), 0, 2)
        
        
        # add line edits to perform cuts on the events
        self.mintime = QtGui.QSpinBox()
        self.mintime_label = QtGui.QLabel("Minimum time between\n two pulses (in ns)")
        self.mintime.setMaximum(2000)
        self.mintime.setValue(400)
        #self.mintime.setValidator(QtGui.QIntValidator())
        self.mintime.setToolTip(QtCore.QString("Reject events where the double pulses are too close together"))
        #self.mintime.setMaxLength(5)
        #self.mintime.setMaximumWidth(60)
        grid.addWidget(self.mintime,2,0)
        grid.addWidget(self.mintime_label,2,1)


        pulsewidthgroupBox = QtGui.QGroupBox("Set conditions on pulse width")
        pulsewidthgroupBox.setCheckable(True)
        pulsewidthgroupBox.setChecked(False)
        pulsewidthgroupBox.setObjectName("pulsewidthgroupbox")      
        pulsewidthlayout = QtGui.QGridLayout(pulsewidthgroupBox)
        
        self.minsinglepulsewidth = QtGui.QSpinBox()
        self.minsinglepulsewidth.setObjectName("minsinglepulsewidth")
        self.minsinglepulsewidth_label = QtGui.QLabel("Min mu pulse width")
        self.minsinglepulsewidth.setSuffix(' ns')
        self.minsinglepulsewidth.setValue(10)
        self.minsinglepulsewidth.setToolTip(QtCore.QString("Define a MINIMUM width for the MUON pulse"))
        self.minsinglepulsewidth.setMaximum(11000)
        pulsewidthlayout.addWidget(self.minsinglepulsewidth,0,0)
        pulsewidthlayout.addWidget(self.minsinglepulsewidth_label,0,1)
 
        self.maxsinglepulsewidth = QtGui.QSpinBox()
        self.maxsinglepulsewidth.setObjectName("maxsinglepulsewidth")
        self.maxsinglepulsewidth_label = QtGui.QLabel("Max mu pulse width")
        self.maxsinglepulsewidth.setSuffix(' ns')
        self.maxsinglepulsewidth.setMaximum(11000)
        self.maxsinglepulsewidth.setValue(300)
        self.maxsinglepulsewidth.setToolTip(QtCore.QString("Define a MAXIMUM width for the MUON pulse"))
        pulsewidthlayout.addWidget(self.maxsinglepulsewidth,1,0)
        pulsewidthlayout.addWidget(self.maxsinglepulsewidth_label,1,1)

        self.mindoublepulsewidth = QtGui.QSpinBox()
        self.mindoublepulsewidth.setObjectName("mindoublepulsewidth")
        self.mindoublepulsewidth_label = QtGui.QLabel("Min e pulse width")
        self.mindoublepulsewidth.setSuffix(' ns')
        self.maxsinglepulsewidth.setMaximum(11000)
        self.mindoublepulsewidth.setValue(5)
        self.mindoublepulsewidth.setToolTip(QtCore.QString("Define a MINIMUM width for the ELECTRON pulse"))
        #self.mintime.setMaximumWidth(60)
        pulsewidthlayout.addWidget(self.mindoublepulsewidth,2,0)
        pulsewidthlayout.addWidget(self.mindoublepulsewidth_label,2,1)

        self.maxdoublepulsewidth = QtGui.QSpinBox()
        self.maxdoublepulsewidth.setObjectName("maxdoublepulsewidth")
        self.maxdoublepulsewidth_label = QtGui.QLabel("Max e pulse width")
        self.maxdoublepulsewidth.setSuffix(' ns')
        self.maxsinglepulsewidth.setMaximum(11000)
        self.maxdoublepulsewidth.setValue(300)
        self.maxdoublepulsewidth.setToolTip(QtCore.QString("Define a MINIMUM width for the ELECTRON pulse"))
        #self.mintime.setMaximumWidth(60)
        pulsewidthlayout.addWidget(self.maxdoublepulsewidth,3,0)
        pulsewidthlayout.addWidget(self.maxdoublepulsewidth_label,3,1)

        grid.addWidget(pulsewidthgroupBox,3,0,1,3)
        self.buttonBox = self.createButtonBox(leftoffset=200)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
        QtCore.QMetaObject.connectSlotsByName(self)
        
        grid.addWidget(self.buttonBox,4,2)
        self.setLayout(grid)
        self.show()
class FitRangeConfigDialog(MuonicDialog):

        def __init__(self, upperlim = None, lowerlim = None, dimension = '', *args):
            QtGui.QDialog.__init__(self,*args)

            #self.resize(480, 360)
            self.setModal(True)
            self.setWindowTitle("Fit Range Configuration")  

            grid = QtGui.QGridLayout()
            
            lower_label    = QtGui.QLabel("Lower limit for the fit range: ") 
            lower = QtGui.QDoubleSpinBox()
            lower.setDecimals(2)
            lower.setSingleStep(0.01)
            lower.setObjectName("lower_limit")
            lower.setSuffix(' %s' %(str(dimension)))
            if lowerlim:
                lower.setMaximum(lowerlim[1])
                lower.setMinimum(lowerlim[0])
                lower.setValue(lowerlim[2])
            grid.addWidget(lower_label,0,0)
            grid.addWidget(lower,0,1)
           
            upper_label    = QtGui.QLabel("Upper limit for the fit range: ") 
            upper = QtGui.QDoubleSpinBox()
            upper.setDecimals(2)
            upper.setSingleStep(0.01)
            upper.setObjectName("upper_limit")
            upper.setSuffix(' %s' %(str(dimension)))            
            if upperlim:
                upper.setMaximum(upperlim[1])
                upper.setMinimum(upperlim[0])
                upper.setValue(upperlim[2])
            grid.addWidget(upper_label,1,0)
            grid.addWidget(upper,1,1)
           
            self.buttonBox = self.createButtonBox(leftoffset=200)
            QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
            QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
            QtCore.QMetaObject.connectSlotsByName(self)
            
            grid.addWidget(self.buttonBox,2,0,2,0)
            self.setLayout(grid)
            self.show()

class VelocityConfigDialog(MuonicDialog):

    def __init__(self, *args):

        QtGui.QDialog.__init__(self,*args)

        #self.resize(480, 360)
        self.setModal(True)
        self.setWindowTitle("Muon Velocity Configuration")  

        grid = QtGui.QGridLayout()
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Upper Channel",objectname = "uppercheckbox",leftoffset=20, setchecked=[0]), 0, 0)
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Lower Channel",objectname = "lowercheckbox",leftoffset=180,setchecked=[1]), 0, 1)
       
        self.buttonBox = self.createButtonBox(leftoffset=200)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
        QtCore.QMetaObject.connectSlotsByName(self)
        
        
        grid.addWidget(self.buttonBox,1,1)
        self.setLayout(grid)
        self.show()

class PeriodicCallDialog(MuonicDialog):
    """
    Issue a command periodically
    """

    def __init__(self, *args):
        QtGui.QDialog.__init__(self,*args)
        self.setModal(True)
        self.v_box = QtGui.QVBoxLayout()
        self.textbox = QtGui.QLineEdit()
        self.time_box = QtGui.QSpinBox()
        self.time_box.setMaximum(600)
        self.time_box.setMinimum(1)
        self.time_box.setSingleStep(1)
        self.command_label = QtGui.QLabel("Command")
        self.interval_label = QtGui.QLabel("Interval")
        self.button_box = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
        self.v_box.addWidget(self.command_label)
        self.v_box.addWidget(self.textbox)
        self.v_box.addWidget(self.interval_label)
        self.v_box.addWidget(self.time_box)
        self.v_box.addWidget(self.button_box)
        
        self.setLayout(self.v_box)
        QtCore.QObject.connect(self.button_box,
                              QtCore.SIGNAL('accepted()'),
                               self.accept
                              )
        QtCore.QObject.connect(self.button_box,
                              QtCore.SIGNAL('rejected()'),
                              self.reject)
        windowtitle = QtCore.QString("PeridicCall")
 
        self.setWindowTitle(windowtitle)
        self.show()


class ThresholdDialog(MuonicDialog):
    """
    Set the Thresholds
    """

    def __init__(self,thr0,thr1,thr2,thr3, *args):

        QtGui.QDialog.__init__(self,*args)

        #dimension = QtCore.QSize()

        #self.resize(260, 260)
        #self.resize(dimension)     
        self.setModal(True)

        self.v_box = QtGui.QVBoxLayout()

        for ch,thr in enumerate([thr0,thr1,thr2,thr3]):
            ch = str(ch)
            thres = QtGui.QSpinBox()
            thres.setMaximum(1000)
            thres.setObjectName("thr_ch_" + ch)
            thres.setValue(int(thr))
            thres.setSuffix(' mV')
            label = QtGui.QLabel("Channel " + ch )
            self.v_box.addWidget(label)
            self.v_box.addWidget(thres)
                        
        self.button_box = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
        self.v_box.addWidget(self.button_box)
        self.setLayout(self.v_box)
        QtCore.QObject.connect(self.button_box,
                              QtCore.SIGNAL('accepted()'),
                               self.accept
                              )

        QtCore.QObject.connect(self.button_box,
                              QtCore.SIGNAL('rejected()'),
                              self.reject)

        windowtitle = QtCore.QString("Thresholds")
        
        self.setWindowTitle(windowtitle)
        self.adjustSize()
        self.show()

class ConfigDialog(MuonicDialog):
    """
    Set Channel configuration
    """
    
    def __init__(self,channelcheckbox_0 = True,channelcheckbox_1 = True,channelcheckbox_2 = True,channelcheckbox_3 = True,coincidencecheckbox_0 = True,coincidencecheckbox_1 = False,coincidencecheckbox_2 = False,coincidencecheckbox_3 = False,vetocheckbox = False,vetocheckbox_0 = False,vetocheckbox_1 = False,vetocheckbox_2 = False, *args):

        QtGui.QDialog.__init__(self,*args)

        self.setObjectName("Configure")
        self.setModal(True)
        self.setWindowTitle("Channel Configuration")  
        self.buttonBox = self.createButtonBox(leftoffset=30, topoffset=300)

        # used advanced grid layout...
        grid = QtGui.QGridLayout()
        channels = []
        if channelcheckbox_0: channels.append(0)
        if channelcheckbox_1: channels.append(1)
        if channelcheckbox_2: channels.append(2)
        if channelcheckbox_3: channels.append(3)
        coincidence = []

        if coincidencecheckbox_0: coincidence.append(0)
        if coincidencecheckbox_1: coincidence.append(1)
        if coincidencecheckbox_2: coincidence.append(2)
        if coincidencecheckbox_3: coincidence.append(3)
        vetochecks = []
        if vetocheckbox_0: vetochecks.append(0)
        if vetocheckbox_1: vetochecks.append(1)
        if vetocheckbox_2: vetochecks.append(2)
        
        grid.addWidget(self.createCheckGroupBox(label="Select Channel",objectname = "channelcheckbox",leftoffset=300,setchecked=channels), 0, 0)
        grid.addWidget(self.createCheckGroupBox(radio=True,label="Trigger Condition",objectname = "coincidencecheckbox",leftoffset=20,setchecked=coincidence,itemlabels=["Single","Twofold","Threefold","Fourfold"]), 0, 1)
        grid.addWidget(self.createCheckGroupBox(radio=True,checkable=True,checkable_set=vetocheckbox,label="Veto",objectname = "vetocheckbox",leftoffset=180,setchecked=vetochecks,itemlabels=["Chan1","Chan2","Chan3"]), 0, 2)
                
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
        QtCore.QMetaObject.connectSlotsByName(self)
        grid.addWidget(self.buttonBox,1,2,1,2)

        self.setLayout(grid)
        
        self.show()

class AdvancedDialog(MuonicDialog):
    """
    Set Configuration dialog
    """
    
    def __init__(self,gatewidth = 100, timewindow = 5.0, nostatus = None, *args):

        QtGui.QDialog.__init__(self,*args)

        self.setObjectName("Configure")
        self.setModal(True)
        self.setWindowTitle("Advanced Configurations")  
        self.buttonBox = self.createButtonBox(leftoffset=30, topoffset=300)

        # used advanced grid layout...
        grid = QtGui.QGridLayout()
        self.gatewidth = QtGui.QSpinBox()
        self.gatewidth.setSuffix(' ns')
        self.gatewidth.setObjectName("gatewidth")
        self.gatewidth_label = QtGui.QLabel("Gatewidth Timewindow (default: 100 ns): ")
        self.gatewidth.setMaximum(159990)
        self.gatewidth.setSingleStep(10)        
        self.gatewidth.setMinimum(10)
        self.gatewidth.setValue(gatewidth)
        self.gatewidth.setToolTip(QtCore.QString("Define a gatewidth, which is the timewindow opend by a trigger"))
        grid.addWidget(self.gatewidth,0,1)
        grid.addWidget(self.gatewidth_label,0,0)

        self.timewindow_label = QtGui.QLabel("Readout Interval (default: 5 s): ")
        self.timewindow = QtGui.QDoubleSpinBox()
        self.timewindow.setDecimals(1)
        self.timewindow.setSingleStep(0.1)
        self.timewindow.setSuffix(' s')
        self.timewindow.setObjectName("timewindow")
        self.timewindow.setMaximum(1000)
        self.timewindow.setMinimum(0.01)
        self.timewindow.setValue(timewindow)
        self.timewindow.setToolTip(QtCore.QString("Define an interval for calculating and refreshing the rates."))
        grid.addWidget(self.timewindow,1,1)
        grid.addWidget(self.timewindow_label,1,0)


        self.nostatus_label = QtGui.QLabel("Write DAQ status lines to RAW file: ")
        self.nostatus = QtGui.QCheckBox()
        self.nostatus.setObjectName("nostatus")
        self.nostatus.setChecked(nostatus)
        self.nostatus.setToolTip(QtCore.QString("Write DAQ status lines to RAW file, same as option -n."))
        self.nostatus.setChecked(True)
        grid.addWidget(self.nostatus,2,1)
        grid.addWidget(self.nostatus_label,2,0)

        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), self.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), self.reject)
        QtCore.QMetaObject.connectSlotsByName(self)
        grid.addWidget(self.buttonBox,3,0,1,2)
        self.setLayout(grid)
        
        self.show()

class HelpDialog(MuonicDialog): 

    def __init__(self, *args):
        _NAME = 'Help'
        QtGui.QDialog.__init__(self,*args)
        self.resize(600, 480)
        self.setModal(True)
        self.setWindowTitle("DAQ Commands")
        self.v_box = QtGui.QVBoxLayout()
        self.textbox = QtGui.QPlainTextEdit(self.helptext())
        self.textbox.setReadOnly(True)
        self.button_box = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok)
        self.v_box.addWidget(self.textbox)
        self.v_box.addWidget(self.button_box)
        self.setLayout(self.v_box)
        QtCore.QObject.connect(self.button_box,
                              QtCore.SIGNAL('accepted()'),
                               self.accept
                              )

        self.show()

    def helptext(self):
        """
        Show this text in the help window
        """
    
        return """QNET2_Vx Help Page
    ------------------------------------------------------------
    Quarknet Scintillator Card 'QNET2_V2' HE=HELP CL=Clear Screen
    Command List Firmware Date 06/23/03 CPU_TempC=31.1 CPU_Volts=3.3
    ------------------------------------------------------------
    Quick Guide,
    see http://neutrino.phys.washington.edu/~walta/qnet_daq/
    for more information
    Manual at
    http://physik-begreifen-zeuthen.desy.de/sites2009/site_PhyBegZ/content/e2198/e2451/e6374/e129804/e129813/infoboxContent129819/DAQ-KarteUserManual_ger.pdf
    ------------------------------------------------------------
    General
    HE - Help
    HE1 - Help page 1
    HEx - Help page x
    ------------------------------------------------------------
    Start & Stop Counter
    CE=Enable
    CD=Disable
    ------------------------------------------------------------
    GPS & Weather
    BA - Barometer Current Reading, Raw BCD counts and kPa.
    TH - Display Digital Thermometer, -40 to 99 degrees C.
    DG - Display GPS Date, Time, Position and Status.
    ------------------------------------------------------------
    Scalars
    DF - Display Scalar Fifo Data, First 12 Bytes.
    DS - Display Scalar Fifo, Counters 0-3, Triggers, and 1PPS Time.
    ------------------------------------------------------------
    Acces internal control registers 
    -(needed to set coincidence and veto criteria as well as gate width)-
    WC MM DD - Write Counter Control Registers CPLD, address MM with DD.
    WT MM DD - Write Time Control Registers TMC address MM with data DD.
    check control registers with
    DC - Display Counters, Control Registers CPLD, address 0-4.
    DT - Display Time Control Registers TMC , address 0-3.
    ---------------------------adresses---------------------------------
    MM - 00 -> 8bits for channel enable/disable, coincidence and veto
    |7   |6   |5          |4          |3       |2       |1       |0       |
    |veto|veto|coincidence|coincidence|channel3|channel2|channel1|channel0|
    ---------------------------bits-------------------------------------
    Set bits for veto:
    ........................
    00 - ch0 is veto
    01 - ch1 is veto
    10 - ch2 is veto
    11 - ch3 is veto
    ........................
    Set bits for coincidence
    ........................ 
    00 - singles
    01 - twofold
    10 - threefold
    11 - fourfold
    --------------------------examples-----------------------------------
    threefold coincidence with veto ch3, all channels enabled
    11101111 -> 1110 1111 two binary numbers -> e f two corresponding hex numbers
    => WC 00 EF should do the job
    ..................................
    no veto, coincidence level twofold, all channels enabled
    00011111 -> 0001 1111 two binary numbers -> 1 f two corresponding hex numbers
    => WC 00 1F should do the job
    .................................
    no veto, single mode, all channels enabled
    00001111 -> 0000 1111 two binary numbers -. 0 f two corresponding hex numbers
    => WC 00 0F should do the job
    
    ...for more descriptions see 
    http://neutrino.phys.washington.edu/~berns/WALTA/Qnet2/FNAL_files/QnetCpldII.doc
    ===================================ADVANCED=======================================
    BC BB.B GG.G - Barometer Calibrate, Store Baseline and Gain, HELP 'HB'.
    NM N - NMEA GPS Data Echo (N==1 On),(N!=1 Off), (GPS_Baud=9600).
    NA N - NMEA GPS Data Append (N==1 On),(N!=1 Off), add GPS to output.
    ST N - Send Status Data (N==1 On),(N!=1 Off), HELP 'HF'.
    GP - Send GPS NMEA Rate GGA=1Sec, RMC=1Sec, others=disable.
    COUNTER - CE=Enable, CD=Disable, Controls TMC Running bit CPLD CCR1.
    3
    FLASH - FL=Load Binary File, FR=Read SumCheck, FC=Copy_2_CPLD.
    RESET - RB=TMC and Counter CPLD, RE=MSP430+TMC+CPLD.
    HELP - HF=Trigger format, HS=Status format, HB=Barometer format
    'HF' Help command on trigger data format
     Timer Counter Bits 31..0 ( 8 bytes ascii)
     RE0 TAG RE0 DATA ( 2 bytes ascii) -- '0x80=Trig_Tag'
     FE0 TAG FE0 DATA ( 2 bytes ascii) -- '0x20=Edge_Tag'
     RE1 TAG RE1 DATA ( 2 bytes ascii) -- '0x1F=Data,(5 bits)'
     FE1 TAG FE1 DATA ( 2 bytes ascii) --
     RE2 TAG RE2 DATA ( 2 bytes ascii) --
     FE2 TAG FE2 DATA ( 2 bytes ascii) --
     RE3 TAG RE3 DATA ( 2 bytes ascii) --
     FE3 TAG FE3 DATA ( 2 bytes ascii) --
     1PPS TIME Bits 31..0 ( 8 bytes ascii)
     GPS RMC UTC hhmmss.sss (10 bytes ascii)
     GPS RMC DATE ddmmyy ( 6 bytes ascii)
     GPS RMC STATUS A=valid ( 1 byte ascii)
     GPS GGA SATELLITES USED ( 2 bytes ascii)
     TRIG IRQ STATUS FLAGS ( 1 byte ascii)
     GPS DATA TO 1PPS DELAY ms ( 5 bytes ascii)
    Status Line Format (* BCD1 BCD2 BCD3 BCD4), 5 minute update synced to triggers
     BAR_kPa -> BCD1 / 10
     GPS_DegC -> BCD2 / 10
     CPU_DegC -> BCD3
     CPU_Vcc -> BCD4 * 0.00122 (3.3 Volt Supply)
     (Pending BCD5-BCD8 CPLD register 0-3 setup data)
     (Pending BCD9 for Board Serial Number)
    'HS' Help command on Status data format
     TRIGGER IRQ STATUS BYTE, BIT ASSIGNMENTS (see 'HF' for location on data line)
     0x1 = 1PPS interrupt pending
     0x2 = Trigger interrupt pending
     0x4 = GPS data could be corrupted (write in progress while readout)
     0x8 = Current or last 1PPS tick not within 41666666 +/-50 clock ticks
    'HB' Help command on Barometer data format
     kPa=Baseline+(BAR_ADC/Gain), If Data= 0xFFFF (not calibrated)
     Flash memory display, (Memory Display of Baseline*10) = 0xFFFF
     Flash memory display, (Memory Display of Gain*10) = 0xFFFF"""


if __name__ == "__main__":

    import sys

    app = QtGui.QApplication(sys.argv)
    dialog  = ConfigDialog()
    ddialog = DecayConfigDialog()
    tdialog = ThresholdDialog(42,42,42,42)
    vdialog = VelocityConfigDialog()
    sys.exit(app.exec_())





"""
Provide the different physics widgets
"""


# Qt4 imports
from PyQt4 import QtGui
from PyQt4 import QtCore

#muonic imports
from .LineEdit import LineEdit
from .MuonicPlotCanvases import ScalarsCanvas,LifetimeCanvas,PulseCanvas,VelocityCanvas,PulseWidthCanvas
from .MuonicDialogs import DecayConfigDialog,PeriodicCallDialog, VelocityConfigDialog, FitRangeConfigDialog
from ..analysis.fit import main as fit
from ..analysis.fit import gaussian_fit
from ..analysis.PulseAnalyzer import VelocityTrigger,DecayTriggerThorough
from matplotlib.backends.backend_qt4agg \
import NavigationToolbar2QTAgg as NavigationToolbar

import datetime

import os
import shutil
import numpy as n
import time

tr = QtCore.QCoreApplication.translate

C = 29979245000 # cm/s

class RateWidget(QtGui.QWidget):
    """
    Display rate plot
    """
    def __init__(self,logger,parent = None):
        QtGui.QWidget.__init__(self,parent = parent)
        self.mainwindow = self.parentWidget()
        self.logger           = logger
        self.run              = False
        self.scalers_result   = False
        self.MAXLENGTH        = 40
        self.scalers_monitor  = ScalarsCanvas(self, logger, self.MAXLENGTH)
        self.rate_mes_start   = datetime.datetime.now()
        self.previous_ch_counts = {"ch0" : 0 ,"ch1" : 0,"ch2" : 0,"ch3": 0}
        self.ch_counts = {"ch0" : 0 ,"ch1" : 0,"ch2" : 0,"ch3": 0}
        self.now = datetime.datetime.now()
        # define the begin of the timeintervall
        # for the rate calculation
        self.lastscalarquery = 0
        self.thisscalarquery = time.time()
        self.timewindow = 0
        self.do_not_show_trigger = False
        # prepare fields for scalars
        self.scalars_ch0_previous = 0
        self.scalars_ch1_previous = 0
        self.scalars_ch2_previous = 0
        self.scalars_ch3_previous = 0
        self.scalars_trigger_previous = 0
        self.scalars_time = 0
        self.start_button = QtGui.QPushButton(tr('MainWindow', 'Start run'))
        self.stop_button = QtGui.QPushButton(tr('MainWindow', 'Stop run'))
        self.label_mean_rates = QtGui.QLabel(tr('MainWindow','mean rates:'))
        self.label_total_scalers = QtGui.QLabel(tr('MainWindow','total counts:'))
        self.label_started = QtGui.QLabel(tr('MainWindow','started:'))
        self.rates = dict()
        self.rates['rates']= None
        self.rates['rates_buffer'] = dict()
        for ch in ['ch0','ch1','ch2','ch3','l_time','trigger']:
            self.rates['rates_buffer'][ch] = []
        self.rates['label_ch0'] = QtGui.QLabel(tr('MainWindow','channel 0:'))
        self.table = QtGui.QTableWidget(5,2,self)
        self.table.setColumnWidth(0,85)
        self.table.setColumnWidth(1,60)
        self.table.setHorizontalHeaderLabels(["rate [1/s]","counts"])
        self.table.setVerticalHeaderLabels(["channel 0","channel 1","channel 2","channel 3","trigger"])
        self.table.horizontalHeader().setStretchLastSection(True)
        self.scalers = dict()
        self.scalers['scalers_buffer'] = dict()
        for ch in ['ch0','ch1','ch2','ch3','trigger']:
            self.scalers['scalers_buffer'][ch] = 0
        
        for cn in enumerate(['edit_ch0','edit_ch1','edit_ch2','edit_ch3','edit_trigger']):
            self.rates[cn[1]] = QtGui.QTableWidgetItem('--')
            self.scalers[cn[1]] = QtGui.QTableWidgetItem('--')
            self.table.setItem(cn[0], 0, self.rates[cn[1]])
            self.rates[cn[1]].setFlags(QtCore.Qt.ItemIsEditable)
            #self.rates[cn[1]].setFlags(QtCore.Qt.ItemIsSelectable)
            self.rates[cn[1]].setFlags(QtCore.Qt.ItemIsEnabled)
            self.table.setItem(cn[0], 1, self.scalers[cn[1]])
            self.scalers[cn[1]].setFlags(QtCore.Qt.ItemIsEditable)
            self.scalers[cn[1]].setFlags(QtCore.Qt.ItemIsSelectable)
            self.scalers[cn[1]].setFlags(QtCore.Qt.ItemIsEnabled)

        self.general_info = dict()
        self.general_info['label_date'] = QtGui.QLabel(tr('MainWindow',''))
        self.general_info['edit_date'] = QtGui.QLineEdit(self)
        self.general_info['edit_date'].setReadOnly(True)
        self.general_info['edit_date'].setDisabled(True)
        self.general_info['label_daq_time'] = QtGui.QLabel(tr('MainWindow','daq time:'))
        self.general_info['edit_daq_time'] = QtGui.QLineEdit(self)
        self.general_info['edit_daq_time'].setReadOnly(True)
        self.general_info['edit_daq_time'].setDisabled(True)
        self.general_info['label_max_rate'] = QtGui.QLabel(tr('MainWindow','max rate:'))
        self.general_info['edit_max_rate'] = QtGui.QLineEdit(self)
        self.general_info['edit_max_rate'].setReadOnly(True)
        self.general_info['edit_max_rate'].setDisabled(True)
        self.general_info['max_rate'] = 0
        self.general_info['min_rate'] = 0

        self.data_file = open(self.mainwindow.filename, 'w')
        self.data_file.write('chan0 | chan1 | chan2 | chan3 | R0 | R1 | R2 | R3 | trigger | Delta_time \n')
        # always write the rate plot data
        self.data_file_write = False

        QtCore.QObject.connect(self.start_button,
                              QtCore.SIGNAL("clicked()"),
                              self.startClicked
                              )

        QtCore.QObject.connect(self.stop_button,
                              QtCore.SIGNAL("clicked()"),
                              self.stopClicked
                              )
        self.stop_button.setEnabled(False)

        ntb = NavigationToolbar(self.scalers_monitor, self)
        rate_widget = QtGui.QGridLayout(self)
        plotBox = QtGui.QGroupBox("")
        plotBox.setObjectName("plotbox")
        plotlayout = QtGui.QGridLayout(plotBox)
        valueBox = QtGui.QGroupBox("")
        valueBox.setObjectName("valuebox")
        valueBox.setMaximumWidth(500)
        valuelayout = QtGui.QGridLayout(valueBox)
        buttomlineBox = QtGui.QGroupBox("")
        buttomlineBox.setObjectName("valuebox")
        blinelayout = QtGui.QGridLayout(buttomlineBox)
        plotlayout.addWidget(self.scalers_monitor,0,0,1,2)
        valuelayout.addWidget(self.table,0,0,1,2)

        valuelayout.addWidget(self.label_started,1,0)
        #valuelayout.addWidget(self.general_info['label_date'],11,3)
        valuelayout.addWidget(self.general_info['edit_date'],1,1,1,1)
        valuelayout.addWidget(self.general_info['label_daq_time'],2,0)
        valuelayout.addWidget(self.general_info['edit_daq_time'],2,1)
        valuelayout.addWidget(self.general_info['label_max_rate'],3,0)
        valuelayout.addWidget(self.general_info['edit_max_rate'],3,1)

        self.table.setEnabled(False)
        blinelayout.addWidget(ntb,4,0,1,3)
        blinelayout.addWidget(self.start_button,4,3)
        blinelayout.addWidget(self.stop_button,4,4)

        rate_widget.addWidget(valueBox,0,2)
        rate_widget.addWidget(plotBox,0,0,1,2)
        rate_widget.addWidget(buttomlineBox,4,0,1,3)
        self.setLayout(rate_widget)

        self.first_bin = False

    def query_daq_for_scalars(self):
        self.lastscalarquery = self.thisscalarquery
        self.mainwindow.daq.put("DS")
        self.thisscalarquery = time.time()

    def calculate(self):
        """
        Get the rates from the observed counts
        by dividing by the measurement interval
        """
        msg = self.mainwindow.daq_msg
        if not (len(msg) >= 2 and msg[0]=='D' and msg[1] == 'S'):
            return False

        # if this is the first time calculate is called, we want to set all
        # counters to zero. This is the beginning of the first bin.
        if self.first_bin:
            self.logger.debug("Buffering muon counts for the first bin of the rate plot.")

            scalars = msg.split()
            for item in scalars:
                if ("S0" in item) & (len(item) == 11):
                    self.scalars_ch0_previous = int(item[3:],16)
                elif ("S1" in item) & (len(item) == 11):
                    self.scalars_ch1_previous = int(item[3:],16)
                elif ("S2" in item) & (len(item) == 11):
                    self.scalars_ch2_previous = int(item[3:],16)
                elif ("S3" in item) & (len(item) == 11):
                    self.scalars_ch3_previous = int(item[3:],16)
                elif ("S4" in item) & (len(item) == 11):
                    self.scalars_trigger_previous = int(item[3:],16)
                elif ("S5" in item) & (len(item) == 11):
                    scalars_time = float(int(item[3:],16))

            self.first_bin = False
            return True

        scalars = msg.split()
        for item in scalars:
            if ("S0" in item) & (len(item) == 11):
                scalars_ch0 = int(item[3:],16)
            elif ("S1" in item) & (len(item) == 11):
                scalars_ch1 = int(item[3:],16)
            elif ("S2" in item) & (len(item) == 11):
                scalars_ch2 = int(item[3:],16)
            elif ("S3" in item) & (len(item) == 11):
                scalars_ch3 = int(item[3:],16)
            elif ("S4" in item) & (len(item) == 11):
                scalars_trigger = int(item[3:],16)
            elif ("S5" in item) & (len(item) == 11):
                scalars_time = float(int(item[3:],16))
            #else:
                #self.logger.debug("unknown item detected: %s" %item.__repr__())
        self.scalars_diff_ch0     = scalars_ch0     - self.scalars_ch0_previous
        self.scalars_diff_ch1     = scalars_ch1     - self.scalars_ch1_previous
        self.scalars_diff_ch2     = scalars_ch2     - self.scalars_ch2_previous
        self.scalars_diff_ch3     = scalars_ch3     - self.scalars_ch3_previous
        self.scalars_diff_trigger = scalars_trigger - self.scalars_trigger_previous
        self.scalars_ch0_previous     = scalars_ch0
        self.scalars_ch1_previous     = scalars_ch1
        self.scalars_ch2_previous     = scalars_ch2
        self.scalars_ch3_previous     = scalars_ch3
        self.scalars_trigger_previous = scalars_trigger

        time_window = self.thisscalarquery - self.lastscalarquery
        rates = (self.scalars_diff_ch0/time_window,\
                 self.scalars_diff_ch1/time_window,\
                 self.scalars_diff_ch2/time_window,\
                 self.scalars_diff_ch3/time_window,\
                 self.scalars_diff_trigger/time_window,\
                 time_window,\
                 self.scalars_diff_ch0,\
                 self.scalars_diff_ch1,\
                 self.scalars_diff_ch2,\
                 self.scalars_diff_ch3,\
                 self.scalars_diff_trigger)

        self.rates['rates'] = rates
        self.timewindow += time_window

        self.scalers['scalers_buffer']['ch0'] += rates[6]
        self.scalers['scalers_buffer']['ch1'] += rates[7]
        self.scalers['scalers_buffer']['ch2'] += rates[8]
        self.scalers['scalers_buffer']['ch3'] += rates[9]
        self.scalers['scalers_buffer']['trigger'] += rates[10]

        max_rate = max([rates[i] for i in range(5)])
        min_rate = min([rates[i] for i in range(5)])
        if max_rate > self.general_info['max_rate']:
            self.general_info['max_rate'] = max_rate
        if min_rate < self.general_info['min_rate']:
            self.general_info['max_rate'] = min_rate

        #write the rates to data file
        # we have to catch IOErrors, can occur if program is
        # exited
        if self.data_file_write:
            try:
                self.data_file.write('%f %f %f %f %f %f %f %f %f %f \n' % (self.scalars_diff_ch0,\
                                                                           self.scalars_diff_ch1,\
                                                                           self.scalars_diff_ch2,\
                                                                           self.scalars_diff_ch3,\
                                                                           rates[0],\
                                                                           rates[1],\
                                                                           rates[2],\
                                                                           rates[3],\
                                                                           rates[4],\
                                                                           rates[5]))
                self.logger.debug("Rate plot data was written to %s" %self.data_file.__repr__())
            except ValueError:
                self.logger.warning("ValueError, Rate plot data was not written to %s" %self.data_file.__repr__())
        return True # succesful!

    def update(self):
        """
        Display newly available data
        """
        if self.run:
            self.query_daq_for_scalars()
            if self.timewindow == 0:
                return
            self.general_info['edit_daq_time'].setText('%.2f s' %(self.timewindow))
            self.general_info['edit_max_rate'].setText('%.2f 1/s' %(self.general_info['max_rate']))
            if self.mainwindow.channelcheckbox_0:
                self.rates['edit_ch0'].setText('%.2f' %(self.scalers['scalers_buffer']['ch0']/self.timewindow))
                self.scalers['edit_ch0'].setText('%i' %(self.scalers['scalers_buffer']['ch0']))
            else:
                self.rates['edit_ch0'].setText('off')
                self.scalers['edit_ch0'].setText('off')
            if self.mainwindow.channelcheckbox_1:
                self.rates['edit_ch1'].setText('%.2f' %(self.scalers['scalers_buffer']['ch1']/self.timewindow))
                self.scalers['edit_ch1'].setText('%i' %(self.scalers['scalers_buffer']['ch1']))
            else:
                self.rates['edit_ch1'].setText('off')
                self.scalers['edit_ch1'].setText('off')
            if self.mainwindow.channelcheckbox_2:
                self.rates['edit_ch2'].setText('%.2f' %(self.scalers['scalers_buffer']['ch2']/self.timewindow))
                self.scalers['edit_ch2'].setText('%i' %(self.scalers['scalers_buffer']['ch2']))
            else:
                self.rates['edit_ch2'].setText('off')
                self.scalers['edit_ch2'].setText('off')
            if self.mainwindow.channelcheckbox_3:
                self.rates['edit_ch3'].setText('%.2f' %(self.scalers['scalers_buffer']['ch3']/self.timewindow))
                self.scalers['edit_ch3'].setText('%i' %(self.scalers['scalers_buffer']['ch3']))
            else:
                self.rates['edit_ch3'].setText('off')
                self.scalers['edit_ch3'].setText('off')

            if self.do_not_show_trigger:
                self.rates['edit_trigger'].setText('off')
                self.scalers['edit_trigger'].setText('off')
            else:
                self.rates['edit_trigger'].setText('%.2f' %(self.scalers['scalers_buffer']['trigger']/self.timewindow))
                self.scalers['edit_trigger'].setText('%i' %(self.scalers['scalers_buffer']['trigger']))

            self.scalers_monitor.update_plot(self.rates['rates'],\
                                             self.do_not_show_trigger,\
                                             self.mainwindow.channelcheckbox_0,\
                                             self.mainwindow.channelcheckbox_1,\
                                             self.mainwindow.channelcheckbox_2,\
                                             self.mainwindow.channelcheckbox_3)

    @property
    def active(self):
        return self.run # rate widget is always active    

    def startClicked(self):
        """
        start the rate measurement and write a file
        """
        self.start_button.setEnabled(False)
        self.stop_button.setEnabled(True)
        time.sleep(0.2)
        self.table.setEnabled(True)

        self.first_bin = True

        self.logger.debug("Start Button Clicked")
        date = datetime.datetime.now()

        self.timewindow = 0
        for ch in ['ch0','ch1','ch2','ch3','trigger']:
            self.scalers['scalers_buffer'][ch] = 0
        for ch in ['ch0','ch1','ch2','ch3','l_time','trigger']:
            self.rates['rates_buffer'][ch] = []

        comment_file = '# new rate measurement run from: %s\n'\
                %date.strftime('%Y-%m-%d_%H-%M-%S')

        #FIXME: This does not belong here
        #if self.mainwindow.tabwidget.decaywidget.active:
        #    comment_file = '# new decay measurement run from: %i-%i-%i %i-%i-%i\n'\
        #                   %(date.tm_year,date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min,date.tm_sec)
        #if self.mainwindow.tabwidget.velocitywidget.active:
        #    comment_file = '# new velocity measurement run from: %i-%i-%i %i-%i-%i\n'\
        #                   %(date.tm_year,date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min,date.tm_sec)


        self.data_file = open(self.mainwindow.filename, 'a')        
        self.data_file.write(comment_file)
        self.run = True
        self.data_file_write = True
        self.now = datetime.datetime.now()
        
        if not self.mainwindow.channelcheckbox_0:
            self.rates['edit_ch0'].setText('off')
            self.scalers['edit_ch0'].setText('off')
        if not self.mainwindow.channelcheckbox_1:
            self.rates['edit_ch1'].setText('off')
            self.scalers['edit_ch1'].setText('off')
        if not self.mainwindow.channelcheckbox_2:
            self.rates['edit_ch2'].setText('off')
            self.scalers['edit_ch2'].setText('off')
        if not self.mainwindow.channelcheckbox_3:
            self.rates['edit_ch3'].setText('off')
            self.scalers['edit_ch3'].setText('off')
        self.general_info['edit_date'].setDisabled(False)
        self.general_info['edit_daq_time'].setDisabled(False)
        self.general_info['edit_max_rate'].setDisabled(False)
        self.general_info['edit_date'].setText(self.now.strftime('%d.%m.%Y %H:%M:%S'))
        self.general_info['edit_daq_time'].setText('%.2f' %(self.timewindow))
        self.general_info['edit_max_rate'].setText('%.2f' %(self.general_info['max_rate']))
        if self.do_not_show_trigger:
            self.rates['edit_trigger'].setText('off')
            self.scalers['edit_trigger'].setText('off')

        self.scalers_monitor.reset(show_pending=True)
        #time.sleep(3)
        #self.start_button.setEnabled(True)
        
    def stopClicked(self):
        """
        hold the rate measurement plot till buttion is pushed again
        """
        self.start_button.setEnabled(True)
        self.stop_button.setEnabled(False)
        self.table.setEnabled(False)

        self.general_info['edit_date'].setDisabled(True)
        self.general_info['edit_daq_time'].setDisabled(True)
        self.general_info['edit_max_rate'].setDisabled(True)

        self.run = False
        self.data_file_write = False
        date = datetime.datetime.now()
        comment_file = '# stopped run on: %s\n'\
                %date.strftime('%Y-%m-%d_%H-%M-%S')
        if not self.data_file.closed:
            self.data_file.write(comment_file)
            self.data_file.close()


class PulseanalyzerWidget(QtGui.QWidget):
    """
    Provide a widget which is able to show a plot of triggered pulses
    """
    def __init__(self,logger,parent=None):
        QtGui.QWidget.__init__(self,parent=parent)
        self.logger = logger
        self.mainwindow = self.parentWidget()
        self.pulsefile = self.mainwindow.pulseextractor.pulsefile
        self.activatePulseanalyzer = QtGui.QCheckBox(self)
        self.activatePulseanalyzer.setText(tr("Dialog", "Show Oscilloscope and Pulse Width Distribution", None, QtGui.QApplication.UnicodeUTF8))
        self.activatePulseanalyzer.setToolTip(QtCore.QString("The oscilloscope will show the last triggered pulses in the selected time window"))
        self.activatePulseanalyzer.setObjectName("activate_pulseanalyzer")
        grid = QtGui.QGridLayout(self)
        self.pulsecanvas = PulseCanvas(self,logger)
        self.pulsecanvas.setObjectName("pulse_canvas")
        self.pulsewidthcanvas = PulseWidthCanvas(self,logger)
        self.pulsewidthcanvas.setObjectName("pulse_width_canvas")
        ntb = NavigationToolbar(self.pulsecanvas, self)
        ntb2 = NavigationToolbar(self.pulsewidthcanvas, self)
        QtCore.QObject.connect(self.activatePulseanalyzer,
                               QtCore.SIGNAL("clicked()"),
                               self.activatePulseanalyzerClicked
                               )

        grid.addWidget(self.activatePulseanalyzer,0,0,1,2)
        grid.addWidget(self.pulsecanvas,1,0)
        grid.addWidget(ntb,2,0) 
        grid.addWidget(self.pulsewidthcanvas,1,1)
        grid.addWidget(ntb2,2,1)
        self.pulses      = None
        self.pulsewidths = []
        self.active = False

    def calculate(self):
        self.pulses = self.mainwindow.pulses
        if self.pulses is None:
            self.logger.debug("Not received any pulses")
            return None
        # pulsewidths changed because falling edge can be None.
        # pulsewidths = [fe - le for chan in pulses[1:] for le,fe in chan]
        pulsewidths = []
        for chan in self.pulses[1:]:
            for le, fe in chan:
                if fe is not None:
                    pulsewidths.append(fe - le)
                else:
                    pulsewidths.append(0.)
        self.pulsewidths += pulsewidths
        
    def update(self):
        self.pulsecanvas.update_plot(self.pulses)
        self.pulsewidthcanvas.update_plot(self.pulsewidths)
        self.pulsewidths = []

    def activatePulseanalyzerClicked(self):
        """
        Perform extra actions when the checkbox is clicked
        """
        if not self.active:
            self.pulsefile = self.mainwindow.pulseextractor.pulsefile
            self.activatePulseanalyzer.setChecked(True)
            self.active = True
            self.logger.debug("Switching on Pulseanalyzer.")
            self.mainwindow.daq.put("CE")

            self.mainwindow.daq.put('CE')
            if not self.pulsefile:
                self.mainwindow.pulsefilename = \
                        os.path.join(self.mainwindow.DATAPATH,"%s_%s_HOURS_%s%s" %(self.mainwindow.now.strftime('%Y-%m-%d_%H-%M-%S'),
                                                                   "P",
                                                                   self.mainwindow.opts.user[0],
                                                                   self.mainwindow.opts.user[1]) )
                self.mainwindow.pulse_mes_start = self.mainwindow.now
                self.mainwindow.pulseextractor.pulsefile = open(self.mainwindow.pulsefilename,'w')
                self.logger.debug("Starting to write pulses to %s" %self.mainwindow.pulsefilename)
                self.mainwindow.writepulses = True

        else:
            self.logger.debug("Switching off Pulseanalyzer.")
            self.activatePulseanalyzer.setChecked(False)            
            self.active = False

            if not self.pulsefile:
                self.mainwindow.pulsefilename = ''
                self.mainwindow.pulse_mes_start = False
                if self.mainwindow.pulseextractor.pulsefile:
                    self.mainwindow.pulseextractor.pulsefile.close()
                self.mainwindow.pulseextractor.pulsefile = False

class StatusWidget(QtGui.QWidget): # not used yet
    """
    Provide a widget which shows the status informations of the DAQ and the muonic software
    """
    def __init__(self,logger,parent=None):
        QtGui.QWidget.__init__(self,parent=parent)
        self.mainwindow = self.parentWidget()
        self.logger = logger
        self.active = True
        # more functional objects

        self.muonic_stats = dict()
        self.daq_stats = dict()

        self.daq_stats['thresholds'] = list()
        for cnt in range(4):
            self.daq_stats['thresholds'].append('not set yet - click on Refresh.')
        self.daq_stats['active_channel_0'] = None
        self.daq_stats['active_channel_1'] = None
        self.daq_stats['active_channel_2'] = None
        self.daq_stats['active_channel_3'] = None
        self.daq_stats['coincidences'] = 'not set yet - click on Refresh.'
        self.daq_stats['coincidence_timewindow'] = 'not set yet - click on Refresh.'
        self.daq_stats['veto'] = 'not set yet - click on Refresh.'

        self.muonic_stats['start_params'] = 'not set yet - click on Refresh.'
        self.muonic_stats['refreshtime'] = 'not set yet - click on Refresh.'
        self.muonic_stats['open_files'] = 'not set yet - click on Refresh.'
        self.muonic_stats['last_path'] = 'not set yet - click on Refresh.'
        self.muonic_stats['decay_veto'] = 'not set yet - start Muon Decay Measurement.'

        self.label_daq = QtGui.QLabel(tr('MainWindow','Status of the DAQ card:'))
        self.label_thresholds = QtGui.QLabel(tr('MainWindow','Threshold:'))
        self.label_active_channels = QtGui.QLabel(tr('MainWindow','Active channels:'))
        self.label_coincidences = QtGui.QLabel(tr('MainWindow','Trigger condition:'))
        self.label_coincidence_timewindow = QtGui.QLabel(tr('MainWindow','Time window for trigger condition:'))
        self.label_veto = QtGui.QLabel(tr('MainWindow','Veto:'))
        self.decay_label_veto = QtGui.QLabel(tr('MainWindow','Muon Decay Veto:'))
        self.thresholds = []
        for cnt in range(4):
            self.thresholds.append(QtGui.QLineEdit(self))
            self.thresholds[cnt].setReadOnly(True)
            self.thresholds[cnt].setText(self.daq_stats['thresholds'][cnt])
            self.thresholds[cnt].setDisabled(True)
        self.active_channel_0 = QtGui.QLineEdit(self)
        self.active_channel_1 = QtGui.QLineEdit(self)
        self.active_channel_2 = QtGui.QLineEdit(self)
        self.active_channel_3 = QtGui.QLineEdit(self)
        self.active_channel_0.setText('Channel 0')
        self.active_channel_1.setText('Channel 1')
        self.active_channel_2.setText('Channel 2')
        self.active_channel_3.setText('Channel 3')
        self.active_channel_0.setReadOnly(True)
        self.active_channel_0.setEnabled(False)
        self.active_channel_1.setReadOnly(True)
        self.active_channel_1.setEnabled(False)
        self.active_channel_2.setReadOnly(True)
        self.active_channel_2.setEnabled(False)
        self.active_channel_3.setReadOnly(True)
        self.active_channel_3.setEnabled(False)
        self.coincidences = QtGui.QLineEdit(self)
        self.coincidences.setReadOnly(True)
        self.coincidences.setDisabled(True)
        self.coincidences.setText(self.daq_stats['coincidences'])
        self.coincidence_timewindow = QtGui.QLineEdit(self)
        self.coincidence_timewindow.setReadOnly(True)
        self.coincidence_timewindow.setDisabled(True)
        self.coincidence_timewindow.setText(self.daq_stats['coincidence_timewindow'])
        self.veto = QtGui.QLineEdit(self)
        self.veto.setReadOnly(True)
        self.veto.setDisabled(True)
        self.veto.setText(self.daq_stats['veto'])
        self.decay_veto = QtGui.QLineEdit(self)
        self.decay_veto.setReadOnly(True)
        self.decay_veto.setDisabled(True)
        self.decay_veto.setText(self.muonic_stats['decay_veto'])

        self.label_muonic = QtGui.QLabel(tr('MainWindow','Status of Muonic:'))
        self.label_measurements = QtGui.QLabel(tr('MainWindow','Active measurements:'))
        self.label_start_params = QtGui.QLabel(tr('MainWindow','Start parameter:'))
        self.label_refreshtime = QtGui.QLabel(tr('MainWindow','Measurement intervals:'))
        self.label_open_files = QtGui.QLabel(tr('MainWindow','Currently opened files:'))
        self.label_last_path = QtGui.QLabel(tr('MainWindow','Last saved files:'))
        self.start_params = QtGui.QPlainTextEdit()
        self.start_params.setReadOnly(True)
        self.start_params.setDisabled(True)
        self.start_params.document().setMaximumBlockCount(10)
        self.measurements = QtGui.QLineEdit(self)
        self.measurements.setReadOnly(True)
        self.measurements.setDisabled(True)
        self.start_params.setPlainText(self.muonic_stats['start_params'])
        self.refreshtime = QtGui.QLineEdit(self)
        self.refreshtime.setReadOnly(True)
        self.refreshtime.setDisabled(True)
        self.refreshtime.setText(self.muonic_stats['refreshtime'])

        self.open_files = QtGui.QPlainTextEdit()
        self.open_files.setReadOnly(True)
        self.open_files.setDisabled(True)
        self.open_files.setPlainText(self.muonic_stats['open_files'])
        self.open_files.document().setMaximumBlockCount(10)
        #self.last_path = QtGui.QLineEdit(self)
        #self.last_path.setReadOnly(True)
        #self.last_path.setDisabled(True)
        #self.last_path.setText(self.muonic_stats['last_path'])

        self.refresh_button  = QtGui.QPushButton(tr('MainWindow','Refresh'))
        self.refresh_button.setDisabled(True)
        QtCore.QObject.connect(self.refresh_button,
                              QtCore.SIGNAL("clicked()"),
                              self.on_refresh_clicked
                              )

        self.save_button  = QtGui.QPushButton(tr('MainWindow','Save to file'))
        QtCore.QObject.connect(self.save_button,
                              QtCore.SIGNAL("clicked()"),
                              self.on_save_clicked
                              )

        status_layout = QtGui.QGridLayout(self)
        status_layout.addWidget(self.label_daq,0,0)
        status_layout.addWidget(self.label_active_channels,1,0)
        status_layout.addWidget(self.active_channel_0,1,1)
        status_layout.addWidget(self.active_channel_1,1,2)
        status_layout.addWidget(self.active_channel_2,1,3)
        status_layout.addWidget(self.active_channel_3,1,4)
        status_layout.addWidget(self.thresholds[0],2,1)
        status_layout.addWidget(self.label_thresholds,2,0)
        status_layout.addWidget(self.thresholds[1],2,2)
        status_layout.addWidget(self.thresholds[2],2,3)
        status_layout.addWidget(self.thresholds[3],2,4)
        status_layout.addWidget(self.label_coincidences,3,0)
        status_layout.addWidget(self.coincidences,3,1,1,2)
        status_layout.addWidget(self.label_coincidence_timewindow,3,3)
        status_layout.addWidget(self.coincidence_timewindow,3,4)
        status_layout.addWidget(self.label_veto,4,0)
        status_layout.addWidget(self.veto,4,1,1,4)
        status_layout.addWidget(self.decay_label_veto,5,0)
        status_layout.addWidget(self.decay_veto,5,1,1,4)
        nix = QtGui.QLabel(self)        
        status_layout.addWidget(nix,6,0)
        status_layout.addWidget(self.label_muonic,7,0)
        status_layout.addWidget(self.label_measurements,8,0)
        status_layout.addWidget(self.measurements,8,1,1,2)
        status_layout.addWidget(self.label_refreshtime,8,3)
        status_layout.addWidget(self.refreshtime,8,4)
        status_layout.addWidget(self.label_start_params,9,0)
        status_layout.addWidget(self.start_params,9,1,2,4)
        status_layout.addWidget(self.label_open_files,11,0)
        status_layout.addWidget(self.open_files,11,1,2,4)
        #status_layout.addWidget(self.label_last_path,11,1)
        #status_layout.addWidget(self.last_path,11,2,1,8)

        status_layout.addWidget(self.refresh_button,13,0,1,6)
        #status_layout.addWidget(self.save_button,11,2,1,2)

    def on_refresh_clicked(self):
        """
        Refresh the status information
        """
        self.refresh_button.setDisabled(True)        
        self.logger.debug("Refreshing status information.")
        self.mainwindow.daq.put('TL')
        time.sleep(0.5)
        self.mainwindow.daq.put('DC')
        time.sleep(0.5)
        self.active = True
        self.mainwindow.processIncoming()

    def on_save_clicked(self):
        """
        Refresh the status information
        """
        self.logger.debug("Saving status information to file.")
        self.logger.warning('Currently not available!')


    def update(self):
        """
        Fill the status information in the widget.
        """
        self.logger.debug("Refreshing status infos")
        if (self.mainwindow.tabwidget.statuswidget.isVisible()):
            self.muonic_stats['start_params'] = str(self.mainwindow.opts).replace('{', '').replace('}','')
            self.muonic_stats['refreshtime'] = str(self.mainwindow.timewindow)+ ' s'
            self.muonic_stats['last_path'] = 'too'
            if self.mainwindow.tabwidget.decaywidget.active:
                self.muonic_stats['decay_veto'] =\
                    'software veto with channel %d' % (self.mainwindow.tabwidget.decaywidget.vetopulsechannel-1)
            
            self.daq_stats['thresholds'][0] = str(self.mainwindow.threshold_ch0)+ ' mV'
            self.daq_stats['thresholds'][1] = str(self.mainwindow.threshold_ch1)+ ' mV'
            self.daq_stats['thresholds'][2] = str(self.mainwindow.threshold_ch2)+ ' mV'
            self.daq_stats['thresholds'][3] = str(self.mainwindow.threshold_ch3)+ ' mV'
            if not self.mainwindow.vetocheckbox:
                self.daq_stats['veto'] = 'no veto set'
            else:
                if self.mainwindow.vetocheckbox_0:
                    self.daq_stats['veto'] = 'veto with channel 0'
                if self.mainwindow.vetocheckbox_1:
                    self.daq_stats['veto'] = 'veto with channel 1'
                if self.mainwindow.vetocheckbox_2:
                    self.daq_stats['veto'] = 'veto with channel 2'

            self.daq_stats['coincidence_timewindow'] = str(self.mainwindow.coincidence_time) + ' ns'

            self.daq_stats['active_channel_0'] = self.mainwindow.channelcheckbox_0
            self.daq_stats['active_channel_1'] = self.mainwindow.channelcheckbox_1
            self.daq_stats['active_channel_2'] = self.mainwindow.channelcheckbox_2
            self.daq_stats['active_channel_3'] = self.mainwindow.channelcheckbox_3
            if self.mainwindow.coincidencecheckbox_0:
                self.daq_stats['coincidences'] = 'Single Coincidence.'
            elif self.mainwindow.coincidencecheckbox_1:
                self.daq_stats['coincidences'] = 'Twofold Coincidence.'
            elif self.mainwindow.coincidencecheckbox_2:
                self.daq_stats['coincidences'] = 'Threefold Coincidence.'
            elif self.mainwindow.coincidencecheckbox_3:
                self.daq_stats['coincidences'] = 'Fourfold Coincidence.'

            for cnt in range(4):
                self.thresholds[cnt].setDisabled(False)                
                self.thresholds[cnt].setText(self.daq_stats['thresholds'][cnt])
                self.thresholds[cnt].setEnabled(True)

            self.active_channel_0.setEnabled(self.daq_stats['active_channel_0'])
            self.active_channel_1.setEnabled(self.daq_stats['active_channel_1'])
            self.active_channel_2.setEnabled(self.daq_stats['active_channel_2'])
            self.active_channel_3.setEnabled(self.daq_stats['active_channel_3'])
            self.coincidences.setText(self.daq_stats['coincidences'])
            self.coincidences.setEnabled(True)
            self.coincidence_timewindow.setText(self.daq_stats['coincidence_timewindow'])
            self.coincidence_timewindow.setEnabled(True)
            self.veto.setText(self.daq_stats['veto'])
            self.veto.setEnabled(True)
            self.decay_veto.setText(self.muonic_stats['decay_veto'])
            self.decay_veto.setEnabled(True)
            
            self.muonic_stats['open_files'] = str(self.mainwindow.filename)
            if self.mainwindow.tabwidget.daqwidget.write_file:
                self.muonic_stats['open_files'] += ', ' + self.mainwindow.rawfilename
            if self.mainwindow.tabwidget.decaywidget.active:
                self.muonic_stats['open_files'] += ', ' + self.mainwindow.decayfilename
            if self.mainwindow.writepulses:
                self.muonic_stats['open_files'] += ', ' + self.mainwindow.pulsefilename
            self.start_params.setPlainText(self.muonic_stats['start_params'])
            self.start_params.setEnabled(True)
            self.refreshtime.setText(self.muonic_stats['refreshtime'])
            self.refreshtime.setEnabled(True)
            self.open_files.setPlainText(self.muonic_stats['open_files'])
            self.open_files.setEnabled(True)
            #self.last_path.setText(self.muonic_stats['last_path'])
            #self.last_path.setEnabled(True)
            measurements = ''
            if self.mainwindow.tabwidget.ratewidget.active:
                measurements += 'Muon Rates'
            if self.mainwindow.tabwidget.decaywidget.active:
                if len(measurements) > 0:
                    measurements += ', '
                measurements += 'Muon Decay'
                self.decay_veto.setText(self.muonic_stats['decay_veto'])
            if self.mainwindow.tabwidget.velocitywidget.active:
                if len(measurements) > 0:
                    measurements += ', '
                measurements += 'Muon Velocity'
            if self.mainwindow.tabwidget.pulseanalyzerwidget.active:
                if len(measurements) > 0:
                    measurements += ', '
                measurements += 'Pulse Analyzer' 
            self.measurements.setText(measurements)
            self.measurements.setEnabled(True)
            self.start_params.setEnabled(True)

            self.active = False
        else:
            self.logger.debug("Status informations widget not active - ignoring update call.")
        self.refresh_button.setDisabled(False)
        self.active = False

        #FIXME: status widget has no pulsefile!
        #if not self.pulsefile:
        #    self.mainwindow.pulsefilename = ''
        #    self.mainwindow.pulse_mes_start = False
        #    if self.mainwindow.pulseextractor.pulsefile:
        #        self.mainwindow.pulseextractor.pulsefile.close()
        #    self.mainwindow.pulseextractor.pulsefile = False



class VelocityWidget(QtGui.QWidget):

    def __init__(self,logger,parent=None):
        QtGui.QWidget.__init__(self,parent=parent)
        self.logger = logger
        self.mainwindow = self.parentWidget()
        self.upper_channel = 0
        self.lower_channel = 1
        self.trigger = VelocityTrigger(logger)
        self.times = []
        self.active = False
        self.binning = (0.,30,25)
        self.fitrange = (self.binning[0],self.binning[1])

        self.activateVelocity = QtGui.QCheckBox(self)
        self.activateVelocity.setText(tr("Dialog", "Measure Flight Time", None, QtGui.QApplication.UnicodeUTF8))
        self.activateVelocity.setObjectName("activate_velocity")
        self.velocityfit_button = QtGui.QPushButton(tr('MainWindow', 'Fit!')) 
        self.velocityfitrange_button = QtGui.QPushButton(tr('MainWindow', 'Fit Range')) 
        displayMuons                 = QtGui.QLabel(self)
        displayMuons.setObjectName("muoncounter")
        self.muon_counter = 0
        lastdecay = QtGui.QLabel(self)
        lastdecay.setObjectName("lastdecay")
        self.last_muon = None
        activesince = QtGui.QLabel(self)
        activesince.setObjectName("activesince")
        self.active_since = None
        layout = QtGui.QGridLayout(self)
        layout.addWidget(self.activateVelocity,0,0,1,3)
        layout.addWidget(displayMuons, 1,0)
        layout.addWidget(lastdecay, 2,0)
        layout.addWidget(activesince, 3,0)
        self.findChild(QtGui.QLabel,QtCore.QString("muoncounter")).setText(tr("Dialog", "We have detected %d muons "%self.muon_counter ,None, QtGui.QApplication.UnicodeUTF8))
        self.velocitycanvas = VelocityCanvas(self,logger,binning = self.binning)
        self.velocitycanvas.setObjectName("velocity_plot")
        layout.addWidget(self.velocitycanvas,4,0,1,3)
        ntb = NavigationToolbar(self.velocitycanvas, self)
        layout.addWidget(ntb,5,0)
        layout.addWidget(self.velocityfitrange_button,5,1)
        layout.addWidget(self.velocityfit_button,5,2)
        self.velocityfitrange_button.setEnabled(False)
        self.velocityfit_button.setEnabled(False)
        QtCore.QObject.connect(self.activateVelocity,
                               QtCore.SIGNAL("clicked()"),
                               self.activateVelocityClicked
                               )
        
        QtCore.QObject.connect(self.velocityfit_button,
                              QtCore.SIGNAL("clicked()"),
                              self.velocityFitClicked
                              )

        QtCore.QObject.connect(self.velocityfitrange_button,
                              QtCore.SIGNAL("clicked()"),
                              self.velocityFitRangeClicked
                              )
        self.pulsefile = self.mainwindow.pulseextractor.pulsefile
        
    def calculate(self):
        pulses = self.mainwindow.pulses
        if pulses is None:
            return
        flighttime = self.trigger.trigger(pulses,upperchannel=self.upper_channel,lowerchannel=self.lower_channel)
        if flighttime != None and flighttime > 0:
            #velocity = (self.channel_distance/((10**(-9))*flighttime))/C #flighttime is in ns, return in fractions of C
            self.logger.info("measured flighttime %s" %flighttime.__repr__())
            self.times.append(flighttime)
            self.last_muon = datetime.datetime.now()
            self.muon_counter += 1

    #FIXME: we should not name this update since update is already a member
    def update(self):
        self.velocityfitrange_button.setEnabled(True)    
        self.velocityfit_button.setEnabled(True)
        self.findChild(VelocityCanvas,QtCore.QString("velocity_plot")).update_plot(self.times)
        self.times = []
        self.findChild(QtGui.QLabel,QtCore.QString("muoncounter")).setText(tr("Dialog", "We have detected %d muons "%self.muon_counter ,None, QtGui.QApplication.UnicodeUTF8))
        self.findChild(QtGui.QLabel,QtCore.QString("lastdecay")).setText(tr("Dialog", "The last muon was detected at %s"%self.last_muon.strftime('%Y-%m-%d %H:%M:%S') ,None, QtGui.QApplication.UnicodeUTF8))

    def velocityFitRangeClicked(self):
        """
        fit the muon velocity histogram
        """
        config_dialog = FitRangeConfigDialog(upperlim = (0.,60.,self.fitrange[1]), lowerlim = (-1.,60.,self.fitrange[0]), dimension = 'ns')
        rv = config_dialog.exec_()
        if rv == 1:
            upper_limit  = config_dialog.findChild(QtGui.QDoubleSpinBox,QtCore.QString("upper_limit")).value()
            lower_limit  = config_dialog.findChild(QtGui.QDoubleSpinBox,QtCore.QString("lower_limit")).value()
            self.fitrange = (lower_limit,upper_limit)

    def velocityFitClicked(self):
        """
        fit the muon velocity histogram
        """
        self.logger.debug("Using fitrange of %s" %self.fitrange.__repr__())
        fitresults = gaussian_fit(bincontent=n.asarray(self.velocitycanvas.heights),binning = self.binning, fitrange = self.fitrange)
        if not fitresults is None:
            self.velocitycanvas.show_fit(fitresults[0],fitresults[1],fitresults[2],fitresults[3],fitresults[4],fitresults[5],fitresults[6],fitresults[7])


    def activateVelocityClicked(self):
        """
        Perform extra actions when the checkbox is clicked
        """
        if not self.active:
            config_dialog = VelocityConfigDialog()
            rv = config_dialog.exec_()
            if rv == 1:
                self.activateVelocity.setChecked(True)
                self.active_since = datetime.datetime.now()
                self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "The measurement is active since %s"%self.active_since.strftime('%Y-%m-%d %H:%M:%S') ,None, QtGui.QApplication.UnicodeUTF8))

                for chan,ch_label in enumerate(["0","1","2","3"]):
                    if config_dialog.findChild(QtGui.QRadioButton,QtCore.QString("uppercheckbox_" + ch_label )).isChecked():
                        self.upper_channel = chan + 1 # ch index is shifted
                        
                for chan,ch_label in enumerate(["0","1","2","3"]):
                    if config_dialog.findChild(QtGui.QRadioButton,QtCore.QString("lowercheckbox_" + ch_label )).isChecked():
                        self.lower_channel = chan + 1 #
            
                self.logger.info("Switching off decay measurement if running!")
                if self.parentWidget().parentWidget().decaywidget.active:
                    self.parentWidget().parentWidget().decaywidget.activateMuondecayClicked()
                self.active = True
                self.parentWidget().parentWidget().parentWidget().daq.put("CE")
                self.parentWidget().parentWidget().ratewidget.startClicked()
                if not self.pulsefile:
                    self.mainwindow.pulsefilename = \
                            os.path.join(self.mainwindow.DATAPATH,"%s_%s_HOURS_%s%s" %(self.mainwindow.now.strftime('%Y-%m-%d_%H-%M-%S'),
                                                                       "P",
                                                                       self.mainwindow.opts.user[0],
                                                                       self.mainwindow.opts.user[1]) )
                    self.mainwindow.pulse_mes_start = self.mainwindow.now
                    self.mainwindow.pulseextractor.pulsefile = open(self.mainwindow.pulsefilename,'w')
                    self.logger.debug("Starting to write pulses to %s" %self.mainwindow.pulsefilename)
                    self.mainwindow.writepulses = True


            else:
                self.activateVelocity.setChecked(False)
                self.active = False
                self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "" ,None, QtGui.QApplication.UnicodeUTF8))
        else:
            self.activateVelocity.setChecked(False)            
            self.active = False
            self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "" ,None, QtGui.QApplication.UnicodeUTF8))
            if not self.pulsefile:
                self.mainwindow.pulsefilename = ''
                self.mainwindow.pulse_mes_start = False
                if self.mainwindow.pulseextractor.pulsefile:
                    self.mainwindow.pulseextractor.pulsefile.close()
                self.mainwindow.pulseextractor.pulsefile = False

            self.mainwindow.tabwidget.ratewidget.stopClicked()  

class DecayWidget(QtGui.QWidget):
    
    def __init__(self,logger,parent=None):
        QtGui.QWidget.__init__(self,parent=parent) 
        self.logger = logger
        self.mufit_button = QtGui.QPushButton(tr('MainWindow', 'Fit!'))
        self.mainwindow = self.parentWidget()        
        self.mufit_button.setEnabled(False)
        self.decayfitrange_button = QtGui.QPushButton(tr('MainWindow', 'Fit Range')) 
        self.decayfitrange_button.setEnabled(False)
        self.lifetime_monitor = LifetimeCanvas(self,logger)
        self.minsinglepulsewidth = 0
        self.maxsinglepulsewidth = 100000 #inf
        self.mindoublepulsewidth = 0
        self.maxdoublepulsewidth = 100000 #inf
        self.muondecaycounter    = 0
        self.lastdecaytime       = 'None'
        self.pulsefile = self.parentWidget().pulseextractor.pulsefile
            
        self.singlepulsechannel  = 0
        self.doublepulsechannel  = 1
        self.vetopulsechannel    = 2 
        self.decay_mintime       = 0
        self.active              = False
        self.trigger             = DecayTriggerThorough(logger)
        self.decay               = []
        self.mu_file             = open("/dev/null","w") 
        self.dec_mes_start       = None
        self.previous_coinc_time_03 = "00"
        self.previous_coinc_time_02 = "0A"
        self.binning = (0,10,21)
        self.fitrange = (1.5,10.) # ignore first bin because of afterpulses, 
                                  # see https://github.com/achim1/muonic/issues/39 

        QtCore.QObject.connect(self.mufit_button,
                              QtCore.SIGNAL("clicked()"),
                              self.mufitClicked
                              )
        QtCore.QObject.connect(self.decayfitrange_button,
                              QtCore.SIGNAL("clicked()"),
                              self.decayFitRangeClicked
                              )

        ntb1 = NavigationToolbar(self.lifetime_monitor, self)

        # activate Muondecay mode with a checkbox
        self.activateMuondecay = QtGui.QCheckBox(self)
        self.activateMuondecay.setObjectName("activate_mudecay")
        self.activateMuondecay.setText(tr("Dialog", "Check for Decayed Muons", None, QtGui.QApplication.UnicodeUTF8))
        QtCore.QObject.connect(self.activateMuondecay,
                              QtCore.SIGNAL("clicked()"),
                              self.activateMuondecayClicked
                              )
        displayMuons                 = QtGui.QLabel(self)
        displayMuons.setObjectName("muoncounter")
        lastDecay                    = QtGui.QLabel(self)
        lastDecay.setObjectName("lastdecay")
        activesince = QtGui.QLabel(self)
        activesince.setObjectName("activesince")
        self.active_since = None
 
        decay_tab = QtGui.QGridLayout(self)
        decay_tab.addWidget(self.activateMuondecay,0,0)
        decay_tab.addWidget(displayMuons,1,0)
        decay_tab.addWidget(lastDecay,2,0)
        decay_tab.addWidget(activesince, 3,0)
        decay_tab.addWidget(self.lifetime_monitor,4,0,1,2)
        decay_tab.addWidget(ntb1,5,0)
        decay_tab.addWidget(self.mufit_button,5,2)
        decay_tab.addWidget(self.decayfitrange_button,5,1)
        self.findChild(QtGui.QLabel,QtCore.QString("muoncounter")).setText(tr("Dialog", "We have %i decayed muons " %self.muondecaycounter, None, QtGui.QApplication.UnicodeUTF8))
        #self.findChild(QtGui.QLabel,QtCore.QString("lastdecay")).setText(tr("Dialog", "Last detected decay at time %s " %self.lastdecaytime, None, QtGui.QApplication.UnicodeUTF8))
        #self.decaywidget = self.widget(1)

    def calculate(self):
        pulses = self.mainwindow.pulses
        #single_channel = self.singlepulsechannel, double_channel = self.doublepulsechannel, veto_channel = self.vetopulsechannel,mindecaytime = self.decay_mintime,minsinglepulsewidth = minsinglepulsewidth,maxsinglepulsewidth = maxsinglepulsewidth, mindoublepulsewidth = mindoublepulsewidth, maxdoublepulsewidth = maxdoublepulsewidth):
        decay =  self.trigger.trigger(pulses,single_channel = self.singlepulsechannel,double_channel = self.doublepulsechannel, veto_channel = self.vetopulsechannel, mindecaytime= self.decay_mintime,minsinglepulsewidth = self.minsinglepulsewidth,maxsinglepulsewidth = self.maxsinglepulsewidth, mindoublepulsewidth = self.mindoublepulsewidth, maxdoublepulsewidth = self.maxdoublepulsewidth )
        if decay != None:
            when = time.asctime()
            self.decay.append((decay/1000,when))
            #devide by 1000 to get microseconds
            
            self.logger.info('We have found a decaying muon with a decaytime of %f at %s' %(decay,when)) 
            self.muondecaycounter += 1
            self.lastdecaytime = when
      
    def mufitClicked(self):
        """
        fit the muon decay histogram
        """
        fitresults = fit(bincontent=n.asarray(self.lifetime_monitor.heights),binning = self.binning, fitrange = self.fitrange)
        if not fitresults is None:
            self.lifetime_monitor.show_fit(fitresults[0],fitresults[1],\
                                           fitresults[2],fitresults[3],\
                                           fitresults[4],fitresults[5],\
                                           fitresults[6],fitresults[7])

    def update(self):
        if not self.decay:
            return

        self.mufit_button.setEnabled(True)
        self.decayfitrange_button.setEnabled(True)

        decay_times =  [decay_time[0] for decay_time in self.decay]
        self.lifetime_monitor.update_plot(decay_times)
        self.findChild(QtGui.QLabel,QtCore.QString("muoncounter")).setText(tr("Dialog", "We have %i decayed muons " %self.muondecaycounter, None, QtGui.QApplication.UnicodeUTF8))
        self.findChild(QtGui.QLabel,QtCore.QString("lastdecay")).setText(tr("Dialog", "Last detected decay at time %s " %self.lastdecaytime, None, QtGui.QApplication.UnicodeUTF8))
        for muondecay in self.decay:
            #muondecay = self.decay[0]
            muondecay_time = muondecay[1].replace(' ','_')
            self.mu_file.write('Decay ')
            self.mu_file.write(muondecay_time.__repr__() + ' ')
            self.mu_file.write(muondecay[0].__repr__())
            self.mu_file.write('\n')
            self.decay = []


    def decayFitRangeClicked(self):
        """
        fit the muon decay histogram
        """
        config_dialog = FitRangeConfigDialog(upperlim = (0.,10.,self.fitrange[1]), lowerlim = (-1.,10.,self.fitrange[0]), dimension = 'microsecond')
        rv = config_dialog.exec_()
        if rv == 1:
            upper_limit  = config_dialog.findChild(QtGui.QDoubleSpinBox,QtCore.QString("upper_limit")).value()
            lower_limit  = config_dialog.findChild(QtGui.QDoubleSpinBox,QtCore.QString("lower_limit")).value()
            self.fitrange = (lower_limit,upper_limit)

    def activateMuondecayClicked(self):
        """
        What should be done if we are looking for mu-decays?
        """
 
        now = datetime.datetime.now()
        #if not self.mainwindow.mudecaymode:
        if not self.active:
                self.activateMuondecay.setChecked(False)
                # launch the settings window
                config_window = DecayConfigDialog()
                rv = config_window.exec_()
                if rv == 1:
                    self.activateMuondecay.setChecked(True)
                    self.active_since = datetime.datetime.now()
                    chan0_single = config_window.findChild(QtGui.QRadioButton,QtCore.QString("singlecheckbox_0")).isChecked()
                    chan1_single = config_window.findChild(QtGui.QRadioButton,QtCore.QString("singlecheckbox_1")).isChecked()
                    chan2_single = config_window.findChild(QtGui.QRadioButton,QtCore.QString("singlecheckbox_2")).isChecked()
                    chan3_single = config_window.findChild(QtGui.QRadioButton,QtCore.QString("singlecheckbox_3")).isChecked()
                    chan0_double = config_window.findChild(QtGui.QRadioButton,QtCore.QString("doublecheckbox_0")).isChecked()
                    chan1_double = config_window.findChild(QtGui.QRadioButton,QtCore.QString("doublecheckbox_1")).isChecked()
                    chan2_double = config_window.findChild(QtGui.QRadioButton,QtCore.QString("doublecheckbox_2")).isChecked()
                    chan3_double = config_window.findChild(QtGui.QRadioButton,QtCore.QString("doublecheckbox_3")).isChecked()
                    chan0_veto   = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_0")).isChecked()
                    chan1_veto   = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_1")).isChecked()
                    chan2_veto   = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_2")).isChecked()
                    chan3_veto   = config_window.findChild(QtGui.QRadioButton,QtCore.QString("vetocheckbox_3")).isChecked()
                    self.decay_mintime   = int(config_window.mintime.value())
                    if config_window.findChild(QtGui.QGroupBox,QtCore.QString("pulsewidthgroupbox")).isChecked():
                        self.minsinglepulsewidth = int(config_window.findChild(QtGui.QSpinBox,QtCore.QString("minsinglepulsewidth")).value())
                        self.maxsinglepulsewidth = int(config_window.findChild(QtGui.QSpinBox,QtCore.QString("maxsinglepulsewidth")).value())
                        self.mindoublepulsewidth = int(config_window.findChild(QtGui.QSpinBox,QtCore.QString("mindoublepulsewidth")).value())
                        self.maxdoublepulsewidth = int(config_window.findChild(QtGui.QSpinBox,QtCore.QString("maxdoublepulsewidth")).value())
                    
                    self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "The measurement is active since %s"%self.active_since.strftime('%Y-%m-%d %H:%M:%S') ,None, QtGui.QApplication.UnicodeUTF8))
                    for channel in enumerate([chan0_single,chan1_single,chan2_single,chan3_single]):
                        if channel[1]:
                            self.singlepulsechannel = channel[0] + 1 # there is a mapping later from this to an index with an offset
                # FIXME! 
                    for channel in enumerate([chan0_double,chan1_double,chan2_double,chan3_double]):
                        if channel[1]:
                            self.doublepulsechannel = channel[0] + 1 # there is a mapping later from this to an index with an offset

                    for channel in enumerate([chan0_veto,chan1_veto,chan2_veto,chan3_veto]):
                        if channel[1]:
                            self.vetopulsechannel = channel[0] + 1 # there is a mapping later from this to an index with an offset
                    self.logger.info("Switching off velocity measurement if running!")
                    if self.parentWidget().parentWidget().velocitywidget.active:
                        self.parentWidget().parentWidget().velocitywidget.activateVelocityClicked()

                    self.logger.warn("We now activate the Muondecay mode!\n All other Coincidence/Veto settings will be overriden!")

                    self.logger.warning("Changing gate width and enabeling pulses") 
                    self.logger.info("Looking for single pulse in Channel %i" %(self.singlepulsechannel - 1))
                    self.logger.info("Looking for double pulse in Channel %i" %(self.doublepulsechannel - 1 ))
                    self.logger.info("Using veto pulses in Channel %i"        %(self.vetopulsechannel - 1 ))

                    self.mu_label = QtGui.QLabel(tr('MainWindow','Muon Decay measurement active!'))
                    self.parentWidget().parentWidget().parentWidget().statusbar.addPermanentWidget(self.mu_label)

                    self.parentWidget().parentWidget().parentWidget().daq.put("DC")

                    self.parentWidget().parentWidget().parentWidget().daq.put("CE") 
                    self.parentWidget().parentWidget().parentWidget().daq.put("WC 03 04")
                    self.parentWidget().parentWidget().parentWidget().daq.put("WC 02 0A")

                    # this should set the veto to none (because we have a software veto)
                    # and the coinincidence to single, so we take all pulses
                    self.parentWidget().parentWidget().parentWidget().daq.put("WC 00 0F")
                  
                    self.mu_file = open(self.parentWidget().parentWidget().parentWidget().decayfilename,'w')        
                    self.dec_mes_start = now
                    #self.decaywidget.findChild("activate_mudecay").setChecked(True)
                    self.active = True
                    #FIXME: is this intentional?
                    self.parentWidget().parentWidget().ratewidget.startClicked()
                    self.pulsefile = self.mainwindow.pulseextractor.pulsefile
                    if not self.pulsefile:
                        self.mainwindow.pulsefilename = \
                            os.path.join(self.mainwindow.DATAPATH,"%s_%s_HOURS_%s%s" %(self.mainwindow.now.strftime('%Y-%m-%d_%H-%M-%S'),
                                                                       "P",
                                                                       self.mainwindow.opts.user[0],
                                                                       self.mainwindow.opts.user[1]) )
                        self.mainwindow.pulse_mes_start = self.mainwindow.now
                        self.mainwindow.pulseextractor.pulsefile = open(self.mainwindow.pulsefilename,'w')
                        self.logger.debug("Starting to write pulses to %s" %self.mainwindow.pulsefilename)
                        self.mainwindow.writepulses = True

                else:
                    self.activateMuondecay.setChecked(False)
                    self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "" ,None, QtGui.QApplication.UnicodeUTF8))
                    self.active = False

        else:
            reset_time = "WC 03 " + self.previous_coinc_time_03
            self.parentWidget().parentWidget().parentWidget().daq.put(reset_time)
            reset_time = "WC 02 " + self.previous_coinc_time_02
            self.parentWidget().parentWidget().parentWidget().daq.put(reset_time)
            self.logger.info('Muondecay mode now deactivated, returning to previous setting (if available)')
            self.parentWidget().parentWidget().parentWidget().statusbar.removeWidget(self.mu_label)
            #self.parentWidget().parentWidget().parentWidget().mudecaymode = False
            mtime = now - self.dec_mes_start
            mtime = round(mtime.seconds/(3600.),2) + mtime.days *86400
            self.logger.info("The muon decay measurement was active for %f hours" % mtime)
            newmufilename = self.parentWidget().parentWidget().parentWidget().decayfilename.replace("HOURS",str(mtime))
            shutil.move(self.parentWidget().parentWidget().parentWidget().decayfilename,newmufilename)
            #self.parentWidget().parentWidget().parentWidget().daq.put("CD")
            self.active = False
            self.activateMuondecay.setChecked(False)
            self.findChild(QtGui.QLabel,QtCore.QString("activesince")).setText(tr("Dialog", "" ,None, QtGui.QApplication.UnicodeUTF8))
            self.parentWidget().parentWidget().ratewidget.stopClicked()            

class DAQWidget(QtGui.QWidget):

    def __init__(self,logger,parent=None):
        QtGui.QWidget.__init__(self,parent=parent)
        self.mainwindow = self.parentWidget()
        self.write_file      = False
        self.label           = QtGui.QLabel(tr('MainWindow','Command'))
        self.hello_edit      = LineEdit()
        self.hello_button    = QtGui.QPushButton(tr('MainWindow','Send'))
        self.file_button     = QtGui.QPushButton(tr('MainWindow', 'Save RAW-File'))
        self.periodic_button = QtGui.QPushButton(tr('MainWindow', 'Periodic Call'))
        QtCore.QObject.connect(self.hello_button,
                              QtCore.SIGNAL("clicked()"),
                              self.on_hello_clicked
                              )
        QtCore.QObject.connect(self.hello_edit,
                              QtCore.SIGNAL("returnPressed()"),
                              self.on_hello_clicked
                              )
        
        QtCore.QObject.connect(self.file_button,
                                QtCore.SIGNAL("clicked()"),
                                self.on_file_clicked
                                )
        QtCore.QObject.connect(self.periodic_button,
                                QtCore.SIGNAL("clicked()"),
                                self.on_periodic_clicked
                                )
        
        self.text_box = QtGui.QPlainTextEdit()
        self.text_box.setReadOnly(True)
        # only 500 lines history
        self.text_box.document().setMaximumBlockCount(500)
        
        daq_layout = QtGui.QGridLayout(self)
        daq_layout.addWidget(self.text_box,0,0,1, 4)
        daq_layout.addWidget(self.label,1,0)
        daq_layout.addWidget(self.hello_edit,1,1)
        daq_layout.addWidget(self.hello_button,1,2) 
        daq_layout.addWidget(self.file_button,1,2) 
        daq_layout.addWidget(self.periodic_button,1,3)   

    def on_hello_clicked(self):

        """
        send a message to the daq
        """
        text = str(self.hello_edit.displayText())
        if len(text) > 0:
            self.mainwindow.daq.put(str(self.hello_edit.displayText()))
            self.hello_edit.add_hist_item(text)
        self.hello_edit.clear()

    def on_file_clicked(self):
        """
        save the raw daq data to a automatically named file
        """
        self.mainwindow.daq.put("CE")        
        self.outputfile = open(self.mainwindow.rawfilename,"w")
        self.file_label = QtGui.QLabel(tr('MainWindow','Writing to %s'%self.mainwindow.rawfilename))
        self.write_file = True
        self.mainwindow.raw_mes_start = datetime.datetime.now()
        self.mainwindow.statusbar.addPermanentWidget(self.file_label)

    def on_periodic_clicked(self):
        """
        issue a command periodically
        """

        periodic_window = PeriodicCallDialog()
        rv = periodic_window.exec_()
        if rv == 1:
            period = periodic_window.time_box.value() * 1000 #We need the period in milliseconds
            command = str(periodic_window.textbox.text())
            commands = command.split('+')
            def periodic_put():
                for c in commands:
                    self.mainwindow.daq.put(c)
            self.periodic_put = periodic_put
            self.periodic_call_timer = QtCore.QTimer()
            QtCore.QObject.connect(self.periodic_call_timer,
                               QtCore.SIGNAL("timeout()"),
                               self.periodic_put)
            self.periodic_put()
            self.periodic_call_timer.start(period)
            self.periodic_status_label = QtGui.QLabel(tr('MainWindow','%s every %s sec'%(command,period/1000)))
            self.mainwindow.statusbar.addPermanentWidget(self.periodic_status_label)
        else:
            try:
                self.periodic_call_timer.stop()
                self.mainwindow.statusbar.removeWidget(self.periodic_status_label)
            except AttributeError:
                pass

    def calculate(self):
        """
        Function that is called via processincoming. It does:
        - starts file writing stuff
        """
        self.text_box.appendPlainText(str(self.mainwindow.daq_msg))
        if self.write_file:
            #self.daq_file.write(self.mainwindow.daq_msg, status = self.mainwindow.statusline)
            self.write_to_file(str(self.mainwindow.daq_msg))

    def write_to_file(self,msg):
        """
        Write the "RAW" file
        :param msg:
        :return:
        """

        try:
            if self.mainwindow.nostatus:
                fields = msg.rstrip("\n").split(" ")
                if ((len(fields) == 16) and (len(fields[0]) == 8)):
                    self.outputfile.write(str(msg)+'\n')
                else:
                    self.logger.debug("Not writing line '%s' to file because it does not contain trigger data" %msg)
            else:
                self.outputfile.write(str(msg)+'\n')

        except ValueError:
            self.logger.warning('Trying to write on closed file, captured!')




class GPSWidget(QtGui.QWidget):

    def __init__(self,logger,parent=None):

        QtGui.QWidget.__init__(self,parent=parent)
        self.active = False
        self.mainwindow = self.parentWidget()
        self.logger = logger
        self.gps_dump = []
        self.read_lines = 13

        self.label           = QtGui.QLabel(tr('MainWindow','GPS Display:'))
        self.refresh_button  = QtGui.QPushButton(tr('MainWindow','Show GPS'))
        self.save_button     = QtGui.QPushButton(tr('MainWindow', 'Save to File'))

        QtCore.QObject.connect(self.refresh_button,
                              QtCore.SIGNAL("clicked()"),
                              self.on_refresh_clicked
                              )
        QtCore.QObject.connect(self.save_button,
                                QtCore.SIGNAL("clicked()"),
                                self.on_save_clicked
                                )
        self.text_box = QtGui.QPlainTextEdit()
        self.text_box.setReadOnly(True)
        # only 500 lines history
        self.text_box.document().setMaximumBlockCount(500)
        self.status_label = QtGui.QLabel(tr('MainWindow','Status: '))
        self.time_label = QtGui.QLabel(tr('MainWindow','GPS time: '))
        self.satellites_label = QtGui.QLabel(tr('MainWindow','#Satellites: '))
        self.chksum_label = QtGui.QLabel(tr('MainWindow','Checksum: '))
        self.latitude_label = QtGui.QLabel(tr('MainWindow','Latitude: '))
        self.longitude_label = QtGui.QLabel(tr('MainWindow','Longitude: '))
        self.altitude_label = QtGui.QLabel(tr('MainWindow','Altitude: '))
        self.posfix_label = QtGui.QLabel(tr('MainWindow','PosFix: '))
        self.status_box = QtGui.QLabel(tr('MainWindow',' Not read out'))
        self.time_box = QtGui.QLabel(tr('MainWindow','--'))
        self.satellites_box = QtGui.QLabel(tr('MainWindow','--'))
        self.chksum_box = QtGui.QLabel(tr('MainWindow','--'))
        self.latitude_box = QtGui.QLabel(tr('MainWindow','--'))
        self.longitude_box = QtGui.QLabel(tr('MainWindow','--'))
        self.altitude_box = QtGui.QLabel(tr('MainWindow','--'))
        self.posfix_box = QtGui.QLabel(tr('MainWindow','--'))

        gps_layout = QtGui.QGridLayout(self)
        gps_layout.addWidget(self.label,0,0,1,4)
        gps_layout.addWidget(self.status_label,1,0)
        gps_layout.addWidget(self.time_label,2,0)
        gps_layout.addWidget(self.satellites_label,3,0)
        gps_layout.addWidget(self.chksum_label,4,0)
        gps_layout.addWidget(self.latitude_label,1,2)
        gps_layout.addWidget(self.longitude_label,2,2)
        gps_layout.addWidget(self.altitude_label,3,2)
        gps_layout.addWidget(self.posfix_label,4,2)
        gps_layout.addWidget(self.status_box,1,1)
        gps_layout.addWidget(self.time_box,2,1)
        gps_layout.addWidget(self.satellites_box,3,1)
        gps_layout.addWidget(self.chksum_box,4,1)
        gps_layout.addWidget(self.latitude_box,1,3)
        gps_layout.addWidget(self.longitude_box,2,3)
        gps_layout.addWidget(self.altitude_box,3,3)
        gps_layout.addWidget(self.posfix_box,4,3)
        gps_layout.addWidget(self.text_box,6,0,1,4)
        gps_layout.addWidget(self.refresh_button,7,0,1,4) 
        #gps_layout.addWidget(self.save_button,1,2) 

        if self.active:
            self.logger.info("Activated GPS display.")
            self.on_refresh_clicked()

    def on_refresh_clicked(self):
        """
        Display/refresh the GPS information
        """
        self.refresh_button.setEnabled(False)
        self.gps_dump = [] 
        self.logger.info('Reading GPS.')
        self.mainwindow.processIncoming()
        self.switch_active(True)        
        self.mainwindow.daq.put('DG')
        self.mainwindow.processIncoming()
        #for count in range(self.read_lines):
        #    msg = self.mainwindow.inqueue.get(True)
        #    self.gps_dump.append(msg)
        #self.calculate()
        #self.logger.info('GPS readout done.')

    def on_save_clicked(self):
        """
        Save the GPS data to an extra file
        """
        #self.outputfile = open(self.mainwindow.rawfilename,"w")
        #self.file_label = QtGui.QLabel(tr('MainWindow','Writing to %s'%self.mainwindow.rawfilename))
        #self.write_file = True
        #self.mainwindow.raw_mes_start = datetime.datetime.now()
        #self.mainwindow.statusbar.addPermanentWidget(self.file_label)
        self.text_box.appendPlainText('save to clicked - function out of order')        
        self.logger.info("Saving GPS informations still disabled.")


    def switch_active(self, switch = False):
        """
        Switch the GPS activation status.
        """
        if switch is None:
            if self.active:
                self.active = False
            else:
                self.active = True
        else:
            self.active = switch
        return self.active
    
    def calculate(self):
        """
        Readout the GPS information and display it in the tab.
        """
        if len(self.gps_dump) < self.read_lines:
            self.logger.warning('Error retrieving GPS information.')
            return False
        __satellites = 0
        __status = False
        __gps_time = ''
        __latitude = ''
        __longitude = ''
        __altitude = ''
        __posfix = 0
        __chksum = False
        self.refresh_button.setEnabled(True)
        try:

            if str(self.gps_dump[3]).strip().replace('Status:','').strip() == 'A (valid)':
                self.logger.info('Valid GPS signal: found %i ' %(__satellites))
                __status = True
                __satellites = int(str(self.gps_dump[8]).strip().replace('Sats used:', '').strip())
                __posfix = int(str(self.gps_dump[4]).strip().replace('PosFix#:', '').strip())
                __gps_time = str(self.gps_dump[2]).strip().replace('Date+Time:', '').strip()
                if str(self.gps_dump[12]).strip().replace('ChkSumErr:', '').strip() == '0':
                    __chksum = True
                else:
                    __chksum = False
 
                __altitude = str(self.gps_dump[7]).strip().replace('Altitude:', '').strip()

                __latitude = str(self.gps_dump[5]).strip().replace('Latitude:', '').strip()

                __longitude = str(self.gps_dump[6]).strip().replace('Longitude:', '').strip()

            else:
                __status = False
                self.logger.info('Invalid GPS signal.')

            self.gps_dump = []
        except:
            self.logger.warning('Error evaluating GPS information.')
            self.gps_dump = []
            self.switch_active(False)
            return False

        if __status:
            __status = 'Valid'
        else:
            __status = 'Invalid'
        if __chksum:
            __chksum = 'No Error'
        else:
            __chksum = 'Error'
                    
        self.status_box.setText(str(__status))
        self.time_box.setText(str(__gps_time))
        self.satellites_box.setText(str(__satellites))
        self.chksum_box.setText(str(__chksum))
        self.latitude_box.setText(str(__latitude))
        self.longitude_box.setText(str(__longitude))
        self.altitude_box.setText(str(__altitude))
        self.posfix_box.setText(str(__posfix))

        self.text_box.appendPlainText('********************')
        self.text_box.appendPlainText('STATUS     : %s' %(str(__status)))
        self.text_box.appendPlainText('TIME          : %s' %(str(__gps_time)))
        self.text_box.appendPlainText('Altitude     : %s' %(str(__altitude)))
        self.text_box.appendPlainText('Latitude     : %s' %(str(__latitude)))
        self.text_box.appendPlainText('Longitude  : %s' %(str(__longitude)))
        self.text_box.appendPlainText('Satellites    : %s' %(str(__satellites)))
        self.text_box.appendPlainText('Checksum   : %s' %(str(__chksum)))
        self.text_box.appendPlainText('********************')

        self.switch_active(False)
        return True



__version__ = "2.0.28"
__source_location__ = "https://github.com/achim1/muonic/"
__docs_hosted_at__  = "http://achim1.github.io/muonic/"
__manual_hosted_at__ = "http://physik-begreifen-zeuthen.desy.de/angebote/kosmische_teilchen/schuelerexperimente/cosmo_experiment/index_ger.html"



#implemented after http://arxiv.org/abs/physics/0605197

import numpy
import scipy.special

gammaln = scipy.special.gammaln

def optbinsize(data,minbins,maxbins):
    
    N = len(data)
    logp = [0]*(maxbins+1)

    for nbins in xrange(minbins,maxbins+1):
        hist,edges = numpy.histogram(data,bins=nbins)
        part1 = N*numpy.log(nbins) + gammaln(nbins/2.) - gammaln(N+nbins/2.)
        part2 = -nbins * gammaln(0.5) + numpy.sum(gammaln(hist+0.5))
        logp[nbins] = part1 + part2

    maximum = max(logp)
    optbins = logp.index(maximum)
    return optbins

# vim: ai ts=4 sts=4 et sw=4



#! /usr/bin/env python

#from distutils.core import setup
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


import os
import shlex
import subprocess as sub
from glob import glob

from muonic.__version__ import __version__,__source_location__

# build the documentation
#man_make  = "make man -C docs"
#html_make = "make html -C docs"

#man_success  = sub.Popen(shlex.split(man_make),stdout=sub.PIPE).communicate()
#html_success = sub.Popen(shlex.split(html_make),stdout=sub.PIPE).communicate()

datapath = (os.getenv('HOME') + os.sep + 'muonic_data')

setup(name='muonic',
      version=__version__,
      description='Software to work with QNet DAQ cards. This is a TRUNK version',
      long_description='Software is able to manage DAQ comunications and shows e.g. a rate plot...',
      author='Robert Franke,Achim Stoessl,Basho Kaminsky',
      author_email="achim.stoessl@desy.de, basho.kaminsky@desy.de",
      url=__source_location__,
      download_url="pip install muonic",
      # can only be used with setuptools
      #install_requires=['numpy','scipy','pyserial','matplotlib','PyQt'],
      license="GPL",
      platforms=["Ubuntu 12.04"],
      classifiers=[
"License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
"Development Status :: 4 - Beta",
"Intended Audience :: Science/Research",
"Intended Audience :: Education",
"Intended Audience :: Developers",
"Programming Language :: Python :: 2.7",
"Topic :: Scientific/Engineering :: Physics"
      ],
      keywords=["QNET","QuarkNET","Fermilab","DESY","DAQ"],
      packages=['muonic','muonic.analysis','muonic.gui','muonic.daq'],
      scripts=['bin/muonic','bin/which_tty_daq'],
      package_data={'muonic': ['daq/simdaq.txt'],'':['*.txt','*.rst']},
      #package_data={'' : ['docs/*','README'], 'muonic': ['daq/simdaq.txt','daq/which_tty_daq']}, 
      data_files=[(datapath,[])]#,(datapath,["muonic/docs/build/man/muonic.1"]),(os.path.join(datapath,"muonic/docs/html"),glob("muonic/docs/build/html/*html")),(os.path.join(datapath,"muonic/docs/html/_static"),glob("muonic/docs/build/html/_static/*")),(os.path.join(datapath,"muonic/docs/html/_modules"),glob("muonic/docs/build/html/_modules/*html")),(os.path.join(datapath,"muonic/docs/html/_sources"),glob("muonic/docs/build/html/_sources/*html")),(os.path.join(datapath,"muonic/docs/html/_modules/muonic/"), glob("muonic/docs/build/html/_modules/muonic/*html")),(os.path.join(datapath,"muonic/docs/html/_modules/muonic/gui"),glob("muonic/docs/build/html/_modules/muonic/*html")),(os.path.join(datapath,"muonic/docs/html/_modules/muonic/daq"),glob("muonic/docs/build/html/_modules/muonic/daq/*html")),(os.path.join(datapath,"muonic/docs/html/_modules/muonic/analysis"), glob("muonic/docs/build/html/_modules/muonic/analysis/*html"))]
      )

# setting correct permissions of created muonic_data dir
#
#userid = os.stat(os.getenv("HOME"))[4]
#gid = os.stat(os.getenv("HOME"))[5]
#
## if muonic is installed with sudo, the ownership of the files
## has to be changed to the current user.
#if os.geteuid() == 0:
#    cline = "chown -R " + str(gid) + ":" + str(userid) + " " + datapath
#    print cline
#
#    chown_success = sub.Popen(shlex.split(cline),stdout=sub.PIPE).communicate()
#
#print man_success[0]
#print html_success[0]
#if chown_success[1] is None:
#    print "Successfully changed owner of %s to %s" %(datapath,str(userid))
#    print "---------------------------"
#
#if man_success[1] is None:
#    print "Built manpages succesfully"
#    print "---------------------------"
#if html_success[1] is None:
#    print "Buitl html docs succesfully"
#    print "---------------------------"
#
#print "MUONIC succesfully installed!"
